@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,17 +32,17 @@ BROWSER_MODE = 3
 # Messaging Error Codes (result-code=)
 ERROR_SUCCESS = 0
 ERROR_UNKNOWN_ERROR = 1
-ERROR_DEVICE_NOT_FOUND = 2
-ERROR_INVALID_DEVICE_ID = 3
-ERROR_INVALID_DEVICE_URI = 4
+ERROR_DEVICE_NOT_FOUND = 2 # MUD: HPMUD_R_INVALID_DEVICE
+ERROR_INVALID_DEVICE_ID = 3 # MUD: HPMUD_R_INVALID_DESCRIPTOR
+ERROR_INVALID_DEVICE_URI = 4 # MUD: HPMUD_R_INVALID_URI
 ERROR_INVALID_MSG_TYPE = 5
 ERROR_INVALID_DATA_ENCODING = 6
 ERROR_INVALID_CHAR_ENCODING = 7
-ERROR_DATA_LENGTH_EXCEEDS_MAX = 8
+ERROR_DATA_LENGTH_EXCEEDS_MAX = 8 # MUD: HPMUD_R_INVALID_LENGTH
 ERROR_DATA_LENGTH_MISMATCH = 9
 ERROR_DATA_DIGEST_MISMATCH = 10
 ERROR_INVALID_JOB_ID = 11
-ERROR_DEVICE_IO_ERROR = 12
+ERROR_DEVICE_IO_ERROR = 12 # MUD: HPMUD_R_IO_ERROR
 ERROR_STRING_QUERY_FAILED = 14
 ERROR_QUERY_FAILED = 15
 ERROR_GUI_NOT_AVAILABLE = 16
@@ -50,35 +50,38 @@ ERROR_NO_CUPS_DEVICES_FOUND = 17 # deprecated
 ERROR_NO_PROBED_DEVICES_FOUND = 18
 ERROR_INVALID_BUS_TYPE = 19 # operation not supported on bus
 ERROR_BUS_TYPE_CANNOT_BE_PROBED = 20
-ERROR_DEVICE_BUSY = 21
+ERROR_DEVICE_BUSY = 21 # MUD: HPMUD_R_DEVICE_BUSY
 ERROR_NO_DATA_AVAILABLE = 22
 ERROR_INVALID_DEVICEID = 23
 ERROR_INVALID_CUPS_VERSION = 24
 ERROR_CUPS_NOT_RUNNING = 25
 ERROR_DEVICE_STATUS_NOT_AVAILABLE = 26
 ERROR_DATA_IN_SHORT_READ = 27
-ERROR_INVALID_SERVICE_NAME = 28
+ERROR_INVALID_SERVICE_NAME = 28 # MUD: HPMUD_R_INVALID_SN
 ERROR_INVALID_USER_ERROR_CODE = 29
-ERROR_ERROR_INVALID_CHANNEL_ID = 30
-ERROR_CHANNEL_BUSY = 31
+ERROR_ERROR_INVALID_CHANNEL_ID = 30 # MUD: HPMUD_R_INVALID_CHANNEL_ID
+ERROR_CHANNEL_BUSY = 31 # MUD: HPMUD_R_INVALID_STATE
 ERROR_CHANNEL_CLOSE_FAILED = 32
 ERROR_UNSUPPORTED_BUS_TYPE = 33 # bus not supported
 ERROR_DEVICE_DOES_NOT_SUPPORT_OPERATION = 34
 ERROR_INVALID_GUI_NAME = 35
 ERROR_INTERFACE_BUSY = 36
-ERROR_DEVICEOPEN_FAILED_ONE_DEVICE_ONLY = 37
-ERROR_DEVICEOPEN_FAILED_DEV_NODE_MOVED = 38
+ERROR_DEVICEOPEN_FAILED = 37 # MUD: HPMUD_R_INVALID_DEVICE_OPEN
+ERROR_INVALID_DEVNODE = 38 # MUD: HPMUD_R_INVALID_DEVICE_NODE
 ERROR_TEST_EMAIL_FAILED = 39
-ERROR_INVALID_HOSTNAME = 45
-ERROR_INVALID_PORT_NUMBER = 46
-ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE = 47
-ERROR_UNSUPPORTED_MODEL = 48
-ERROR_FAX_FILE_NOT_FOUND = 49
+ERROR_INVALID_HOSTNAME = 45 # MUD: HPMUD_R_INVALID_IP
+ERROR_INVALID_PORT_NUMBER = 46 # MUD: HPMUD_R_INVALID_IP_PORT
+ERROR_INVALID_TIMEOUT = 47 # MUD: HPMUD_R_INVALID_TIMEOUT
+ERROR_DATFILE_ERROR = 48 # MUD: HPMUD_R_DATFILE_ERROR
+ERROR_IO_TIMEOUT = 49 # MUD: HPMUD_R_IO_TIMEOUT 
 ERROR_FAX_INCOMPATIBLE_OPTIONS = 50
 ERROR_FAX_INVALID_FAX_FILE = 51
 ERROR_FAX_MUST_RUN_SENDFAX_FIRST = 52
 ERROR_FAX_PROCESSING = 53
 ERROR_FAX_READY = 54
+ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE = 55
+ERROR_UNSUPPORTED_MODEL = 56
+ERROR_FAX_FILE_NOT_FOUND = 57
 # --> add new codes here <--
 ERROR_UNABLE_TO_BIND_SOCKET = 95
 ERROR_UNABLE_TO_CONTACT_SERVICE = 96
@@ -98,6 +101,7 @@ ERROR_INTERNAL = 99
 
 STATUS_UNKNOWN = 0
 
+EVENT_MIN_USER_EVENT = 500
 EVENT_START_PRINT_JOB = 500 # sent by hp: backend
 EVENT_END_PRINT_JOB = 501 # sent by hp: backend
 EVENT_START_FAX_PRINT_JOB = 600 # sent by hpfax: backend
@@ -220,8 +224,8 @@ EVENT_ERROR_DEVICE_NOT_FOUND = 5002
 EVENT_ERROR_INVALID_DEVICE_ID = 5003
 EVENT_ERROR_INVALID_DEVICE_URI = 5004
 EVENT_ERROR_INVALID_MSG_TYPE = 5005
-EVENT_ERROR_INVALID_DATA_ENCODING = 5006
-EVENT_ERROR_INVALID_CHAR_ENCODING = 5007
+#EVENT_ERROR_INVALID_DATA_ENCODING = 5006
+#EVENT_ERROR_INVALID_CHAR_ENCODING = 5007
 EVENT_ERROR_DATA_LENGTH_EXCEEDS_MAX = 5008
 EVENT_ERROR_DATA_LENGTH_MISMATCH = 5009
 EVENT_ERROR_DATA_DIGEST_MISMATCH = 5010
@@ -248,7 +252,7 @@ EVENT_ERROR_CHANNEL_BUSY = 5031
 EVENT_ERROR_CHANNEL_CLOSE_FAILED = 5032
 EVENT_ERROR_UNSUPPORTED_BUS_TYPE = 5033 # bus not supported
 EVENT_ERROR_DEVICE_DOES_NOT_SUPPORT_OPERATION = 5034
-EVENT_ERROR_INVALID_GUI_NAME = 5035
+#EVENT_ERROR_INVALID_GUI_NAME = 5035
 EVENT_ERROR_INTERFACE_BUSY = 5036
 EVENT_ERROR_DEVICEOPEN_FAILED_ONE_DEVICE_ONLY = 5037
 EVENT_ERROR_DEVICEOPEN_FAILED_DEV_NODE_MOVED = 5038
@@ -269,13 +273,16 @@ EVENT_MAX_USER_EVENT = 7999
 
 # fax (internal events)
 EVENT_FAX_MIN = 8000
-EVENT_FAX_RENDER_COMPLETE = 8000 # sent by hpssd to hp-sendfax after job rendered
-EVENT_FAX_ADDRESS_BOOK_UPDATED = 8002 # Sent by FAB to indicate that the dB has changed
-EVENT_FAX_RENDER_DISTANT_EARLY_WARNING = 8003
+EVENT_FAX_RENDER_COMPLETE = 8000 
+#EVENT_FAX_ADDRESS_BOOK_UPDATED = 8002 # Sent by FAB to indicate that the dB has changed
+#EVENT_FAX_RENDER_DISTANT_EARLY_WARNING = 8003
 EVENT_FAX_MAX = 8999
 # end
 
 EVENT_CUPS_QUEUES_CHANGED = 9000 # sent by hp-setup if queues added (or removed)
+EVENT_RAISE_DEVICE_MANAGER = 9001
+
+EVENT_MAX_EVENT = 9999
 
 # end of events
 
@@ -360,7 +367,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     STATUS_PRINTER_FUSER_ERROR : ERROR_STATE_ERROR,
     STATUS_DEVICE_UNSUPPORTED : ERROR_STATE_ERROR,
     # The following block are EVENTs because they are only
-    # recieved as events from hpiod, hp backend, etc.
+    # recieved as events from hpmud, hp backend, etc.
     # i.e., a device does not produce status codes in this range
     EVENT_ERROR_SUCCESS : ERROR_STATE_CLEAR,
     EVENT_ERROR_UNKNOWN_ERROR : ERROR_STATE_ERROR,
@@ -368,8 +375,8 @@ STATUS_TO_ERROR_STATE_MAP = {
     EVENT_ERROR_INVALID_DEVICE_ID : ERROR_STATE_ERROR,
     EVENT_ERROR_INVALID_DEVICE_URI : ERROR_STATE_ERROR,
     EVENT_ERROR_INVALID_MSG_TYPE : ERROR_STATE_ERROR,
-    EVENT_ERROR_INVALID_DATA_ENCODING : ERROR_STATE_ERROR,
-    EVENT_ERROR_INVALID_CHAR_ENCODING : ERROR_STATE_ERROR,
+    #EVENT_ERROR_INVALID_DATA_ENCODING : ERROR_STATE_ERROR,
+    #EVENT_ERROR_INVALID_CHAR_ENCODING : ERROR_STATE_ERROR,
     EVENT_ERROR_DATA_LENGTH_EXCEEDS_MAX : ERROR_STATE_WARNING,
     EVENT_ERROR_DATA_LENGTH_MISMATCH : ERROR_STATE_ERROR,
     EVENT_ERROR_DATA_DIGEST_MISMATCH : ERROR_STATE_ERROR,
@@ -396,7 +403,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     EVENT_ERROR_CHANNEL_CLOSE_FAILED : ERROR_STATE_ERROR,
     EVENT_ERROR_UNSUPPORTED_BUS_TYPE : ERROR_STATE_ERROR,
     EVENT_ERROR_DEVICE_DOES_NOT_SUPPORT_OPERATION : ERROR_STATE_ERROR,
-    EVENT_ERROR_INVALID_GUI_NAME : ERROR_STATE_ERROR,
+    #EVENT_ERROR_INVALID_GUI_NAME : ERROR_STATE_ERROR,
     EVENT_ERROR_INTERFACE_BUSY : ERROR_STATE_BUSY,
     EVENT_ERROR_DEVICEOPEN_FAILED_ONE_DEVICE_ONLY : ERROR_STATE_ERROR,
     EVENT_ERROR_DEVICEOPEN_FAILED_DEV_NODE_MOVED : ERROR_STATE_ERROR,
@@ -422,6 +429,26 @@ STATUS_TO_ERROR_STATE_MAP = {
     EVENT_PCARD_JOB_FAIL : ERROR_STATE_ERROR,
     EVENT_PCARD_UNABLE_TO_MOUNT : ERROR_STATE_ERROR,
     EVENT_PCARD_FILES_TRANSFERED : ERROR_STATE_OK,
+
+    # Low supplies
+    STATUS_PRINTER_LOW_BLACK_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_TRI_COLOR_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_CYAN_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_MAGENTA_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_YELLOW_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_CYAN_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_MAGENTA_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_YELLOW_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_GRAY_INK : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_PHOTO_BLUE_INK : ERROR_STATE_LOW_SUPPLIES,
+
+    STATUS_PRINTER_LOW_BLACK_TONER : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_CYAN_TONER : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_MAGENTA_TONER : ERROR_STATE_LOW_SUPPLIES,
+    STATUS_PRINTER_LOW_YELLOW_TONER : ERROR_STATE_LOW_SUPPLIES,
+    # end
+
 }
 
 
@@ -545,6 +572,7 @@ COLOR_CAL_TYPE_STRINGRAY_LONGBOW_TORNADO = 3
 COLOR_CAL_TYPE_CONNERY = 4
 COLOR_CAL_TYPE_COUSTEAU = 5
 COLOR_CAL_TYPE_CARRIER = 6
+COLOR_CAL_TYPE_TYPHOON = 7
 
 # status-types
 STATUS_TYPE_NONE = 0
@@ -555,12 +583,15 @@ STATUS_TYPE_LJ = 3
 #STATUS_TYPE_S_SNMP = 5 # DEPRECATED
 STATUS_TYPE_LJ_XML = 6
 #STATUS_TYPE_S_LIDIL = 7 # DEPRECATED
+STATUS_TYPE_PJL = 8
+STATUS_TYPE_PML_AND_PJL = 9 # Same as types 3(tbx)+8(hp:)
 
-# status-battery-check-types
+# status-battery-check
 STATUS_BATTERY_CHECK_NONE = 0
-STATUS_BATTERY_CHECK_STD = 1 # Deskjet 450/460
+STATUS_BATTERY_CHECK_STD = 1 # Deskjet 450/460 (PML or dyn. counters)
+STATUS_BATTERY_CHECK_PML = 2 # Officejet H470 (PML)
 
-# status-dynamic-counters-type
+# status-dynamic-counters
 STATUS_DYNAMIC_COUNTERS_NONE = 0
 STATUS_DYNAMIC_COUNTERS_PCL = 1
 STATUS_DYNAMIC_COUNTERS_PML_SNMP = 2
@@ -584,6 +615,7 @@ FAX_TYPE_NONE = 0
 FAX_TYPE_BLACK_SEND_EARLY_OPEN = 1 # newer models
 FAX_TYPE_BLACK_SEND_LATE_OPEN = 2 # older models
 FAX_TYPE_BLACK_AND_COLOR_SEND = 3 # future
+FAX_TYPE_SOAP = 4
 
 # pcard-types
 PCARD_TYPE_NONE = 0
@@ -595,6 +627,8 @@ SCAN_TYPE_NONE = 0
 SCAN_TYPE_SCL = 1
 SCAN_TYPE_PML = 2
 SCAN_TYPE_SOAP = 3
+SCAN_TYPE_MARVEL = 4
+SCAN_TYPE_SOAP2 = 5
 
 # scan-styles
 SCAN_STYLE_NONE = 0
@@ -646,25 +680,76 @@ IO_SUPPORT_NETWORK = 0x4
 IO_SUPPORT_WIRELESS = 0x8
 IO_SUPPORT_BLUETOOTH = 0x10
 
-# Model categories
-MODEL_TYPE_UNSUPPORTED = -1
-MODEL_TYPE_COLOR_INKJET = 0
-MODEL_TYPE_PHOTOSMART = 1
-MODEL_TYPE_COLOR_LASER = 2
-MODEL_TYPE_COLOR_LASER_AIO = 3
-MODEL_TYPE_COLOR_INKJET_AIO = 4
-MODEL_TYPE_MONO_LASER = 5
-MODEL_TYPE_MONO_LASER_AIO = 6
-MODEL_TYPE_EDGELINE = 7
+### Model categories
+##MODEL_TYPE_UNSUPPORTED = -1
+##MODEL_TYPE_COLOR_INKJET = 0
+##MODEL_TYPE_PHOTOSMART = 1
+##MODEL_TYPE_COLOR_LASER = 2
+##MODEL_TYPE_COLOR_LASER_AIO = 3
+##MODEL_TYPE_COLOR_INKJET_AIO = 4
+##MODEL_TYPE_MONO_LASER = 5
+##MODEL_TYPE_MONO_LASER_AIO = 6
+##MODEL_TYPE_EDGELINE = 7
 
-# 'io-mode'
+# New more user friendly model categories
+MODEL_TYPE2_UNSUPPORTED = 0
+MODEL_TYPE2_DESKJET = 1
+MODEL_TYPE2_DESKJET_AIO = 2
+MODEL_TYPE2_PHOTOSMART = 3
+MODEL_TYPE2_OFFICEJET = 4
+MODEL_TYPE2_PSC = 5
+MODEL_TYPE2_LASERJET = 6
+MODEL_TYPE2_COLOR_LASERJET = 7
+MODEL_TYPE2_BIJ = 8
+MODEL_TYPE2_EDGELINE = 9
+MODEL_TYPE2_APOLLO = 10
+MODEL_TYPE2_SCANJET = 11 # not supported
+MODEL_TYPE2_DESIGNJET = 12 # not supported
+MODEL_TYPE2_OTHER = 13 # No specific "sub-brand"
+
+# 'io-mode' and 'io-mfp-mode'
 IO_MODE_UNI = 0
 IO_MODE_RAW = 1
-IO_MODE_MLC = 2
+#IO_MODE_NOT_USED = 2
 IO_MODE_DOT4 = 3
+IO_MODE_DOT4_PHOENIX = 4
+IO_MODE_DOT4_BRIDGE = 5
+IO_MODE_MLC_GUSHER = 6
+IO_MODE_MLC_MISER = 7
 
+# plugins
+PLUGIN_NONE = 0
+PLUGIN_REQUIRED = 1
+PLUGIN_OPTIONAL = 2
 
+# embedded web server
+EWS_NONE = 0
+EWS_PRESENT = 1
 
+# panel check
+PANEL_CHECK_TYPE_NONE = 0
+PANEL_CHECK_TYPE_CHECK = 1
 
+# pq diag
+PQ_DIAG_TYPE_NONE = 0
+PQ_DIAG_TYPE_1 = 1
+PQ_DIAG_TYPE_2 = 2
 
+# line feed cal
+LINEFEED_CAL_TYPE_NONE = 0
+LINEFEED_CAL_TYPE_OJ_K550 = 1
+LINEFEED_CAL_TYPE_OJ_PRO_L7XXX = 2
+
+# power-settings
+POWER_SETTINGS_NONE = 0
+POWER_SETTINGS_EPML = 1 # DJ450/DJ460
+POWER_SETTINGS_PML = 2 # OJ H470
+
+# Paper types (for hp-toolbox paper prompts)
+PAPER_TYPE_PLAIN = 0
+PAPER_TYPE_HP_ADV_PHOTO = 1
+
+# job-storage
+JOB_STORAGE_DISABLE = 0
+JOB_STORAGE_ENABLE = 1
 
