@@ -27,15 +27,24 @@ from prnt import pcl, ldl, colorcal
 
 # ********************** Align **********************
 
-def AlignType1(dev, loadpaper_ui): # Auto VIP
+def AlignType1(dev, loadpaper_ui): # Auto VIP (using embedded PML)
     ok = loadpaper_ui()
     if ok:
         dev.writeEmbeddedPML(pml.OID_AUTO_ALIGNMENT,
                              pml.AUTO_ALIGNMENT, style=0, 
-                             direct=False)
+                             direct=True)
         dev.closePrint()
 
     return ok
+    
+def AlignType1PML(dev, loadpaper_ui): # Auto VIP (using PML)
+    ok = loadpaper_ui()
+    if ok:
+        dev.setPML(pml.OID_AUTO_ALIGNMENT, pml.AUTO_ALIGNMENT)
+        dev.closePML()
+
+    return ok
+
 
 
 def AlignType2(dev, loadpaper_ui, align_ui, bothpens_ui): # 8xx
@@ -946,13 +955,12 @@ def AlignType12(dev, loadpaper_ui):
 def cleaning(dev, clean_type, level1, level2, level3,
               loadpaper_ui, dlg1, dlg2, dlg3, wait_ui):
 
-    CLEAN_SLEEP_TIMER = 60
     state = 0
 
     while state != -1:
         if state == 0: # Initial level1 print
             state = 1
-            if clean_type == 3:
+            if clean_type == CLEAN_TYPE_PCL_WITH_PRINTOUT:
                 ok = loadpaper_ui()
                 if not ok:
                     state = -1
@@ -1023,7 +1031,6 @@ def print_clean_test_page(dev):
     dev.printGzipFile(os.path.join(prop.home_dir, 'data',
                       'ps', 'clean_page.pdf.gz'), raw=False)
 
-
 def cleanType1(dev): # PCL, Level 1
     dev.writeEmbeddedPML(pml.OID_CLEAN, pml.CLEAN_CLEAN)
     dev.closePrint()
@@ -1381,6 +1388,11 @@ def colorCalType6(dev, loadpaper_ui):
     if loadpaper_ui():
         dev.setPML(pml.OID_PRINT_INTERNAL_PAGE, pml.PRINT_INTERNAL_PAGE_COLOR_CAL)
         dev.closePML()
+        
+def colorCalType7(dev, loadpaper_ui):
+    if loadpaper_ui():
+        dev.setPML(pml.OID_PRINT_INTERNAL_PAGE, pml.PRINT_INTERNAL_PAGE_AUTOMATIC_COLOR_CALIBRATION)
+        dev.closePML()
 
 # ********************** LF Cal **********************        
         
