@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2008 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,12 +29,58 @@ import fcntl, errno, stat, string, commands
 import cStringIO, re
 import xml.parsers.expat as expat
 import getpass
+import locale
 
+try:
+    import platform
+    platform_avail = True
+except ImportError:
+    platform_avail = False
+    
 # Local
 from g import *
 from codes import *
 import pexpect
 
+
+
+def lock(f):
+    try:
+        fcntl.flock(f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
+        return True
+    except IOError:
+        return False
+
+        
+def unlock(f):
+    try:
+        fcntl.flock(f.fileno(), fcntl.LOCK_UN)
+    except IOError:
+        pass
+        
+        
+def lock_app(application, suppress_error=False):
+    if not os.path.exists(prop.user_dir):
+        os.makedirs(prop.user_dir)
+        
+    lock_file = os.path.join(prop.user_dir, '.'.join([application, 'lock']))
+    try:
+        lock_file_f = open(lock_file, "w")
+    except IOError:
+        if not suppress_error:
+            log.error("Unable to open %s lock file." % lock_file) 
+        return False, None
+
+    log.debug("Locking file: %s" % lock_file)
+    
+    if not lock(lock_file_f):
+        if not suppress_error:
+            log.error("Unable to lock %s. Is %s already running?" % (lock_file, application))
+        return False, None
+    
+    return True, lock_file_f
+
+
 xml_basename_pat = re.compile(r"""HPLIP-(\d*)_(\d*)_(\d*).xml""", re.IGNORECASE)
 
 
@@ -53,113 +99,57 @@ def Translator(frm='', to='', delete='', keep=None):
 
     return callable
 
-# For pidfile locking (must be "static" and global to the whole app)
-prv_pidfile = None
-prv_pidfile_name = ""
-
-
-def get_pidfile_lock (a_pidfile_name=""):
-    """ Call this to either lock the pidfile, or to update it after a fork()
-        Credit: Henrique M. Holschuh <hmh@debian.org>
-    """
-    global prv_pidfile
-    global prv_pidfile_name
-    if prv_pidfile_name == "":
-        try:
-            prv_pidfile_name = a_pidfile_name
-            prv_pidfile = os.fdopen(os.open(prv_pidfile_name, os.O_RDWR | os.O_CREAT, 0644), 'r+')
-            fcntl.fcntl(prv_pidfile.fileno(), fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-            while 1:
-                try:
-                    fcntl.flock(prv_pidfile.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
-                except (OSError, IOError), e:
-                    if e.errno == errno.EINTR:
-                        continue
-                    elif e.errno == errno.EWOULDBLOCK:
-                        try:
-                            prv_pidfile.seek(0)
-                            otherpid = int(prv_pidfile.readline(), 10)
-                            sys.stderr.write ("can't lock %s, running daemon's pid may be %d\n" % (prv_pidfile_name, otherpid))
-                        except (OSError, IOError), e:
-                            sys.stderr.write ("error reading pidfile %s: (%d) %s\n" % (prv_pidfile_name, e.errno, e.strerror))
-
-                        sys.exit(1)
-                    sys.stderr.write ("can't lock %s: (%d) %s\n" % (prv_pidfile_name, e.errno, e.strerror))
-                    sys.exit(1)
-                break
-        except (OSError, IOError), e:
-            sys.stderr.write ("can't open pidfile %s: (%d) %s\n" % (prv_pidfile_name, e.errno, e.strerror))
-            sys.exit(1)
-    try:
-        prv_pidfile.seek(0)
-        prv_pidfile.write("%d\n" % (os.getpid()))
-        prv_pidfile.flush()
-        prv_pidfile.truncate()
-    except (OSError, IOError), e:
-        log.error("can't update pidfile %s: (%d) %s\n" % (prv_pidfile_name, e.errno, e.strerror))
+##def daemonize (stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
+##    """
+##    Credit: Jürgen Hermann, Andy Gimblett, and Noah Spurrier
+##            http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66012
+##    """
+##
+##    # Do first fork.
+##    try:
+##        pid = os.fork()
+##        if pid > 0:
+##            sys.exit(0) # Exit first parent.
+##    except OSError, e:
+##        sys.stderr.write ("fork #1 failed: (%d) %s\n" % (e.errno, e.strerror))
+##        sys.exit(1)
+##
+##    # Decouple from parent environment.
+##    os.chdir("/")
+##    os.umask(0)
+##    os.setsid()
+##
+##    # Do second fork.
+##    try:
+##        pid = os.fork()
+##        if pid > 0:
+##            sys.exit(0) # Exit second parent.
+##    except OSError, e:
+##        sys.stderr.write ("fork #2 failed: (%d) %s\n" % (e.errno, e.strerror))
+##        sys.exit(1)
+##
+##    # Now I am a daemon!
+##    # Redirect standard file descriptors.
+##    si = file(stdin, 'r')
+##    so = file(stdout, 'a+')
+##    se = file(stderr, 'a+', 0)
+##    os.dup2(si.fileno(), sys.stdin.fileno())
+##    os.dup2(so.fileno(), sys.stdout.fileno())
+##    os.dup2(se.fileno(), sys.stderr.fileno())
 
 
 
-def daemonize (stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
-    """
-    Credit: Jürgen Hermann, Andy Gimblett, and Noah Spurrier
-            http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66012
-
-    Proper pidfile support: Henrique M. Holschuh <hmh@debian.org>
-    """
-    # Try to lock pidfile if not locked already
-    if prv_pidfile_name != '' or prv_pidfile_name != "":
-        get_pidfile_lock(prv_pidfile_name)
-
-    # Do first fork.
-    try:
-        pid = os.fork()
-        if pid > 0:
-            sys.exit(0) # Exit first parent.
-    except OSError, e:
-        sys.stderr.write ("fork #1 failed: (%d) %s\n" % (e.errno, e.strerror))
-        sys.exit(1)
-
-    # Decouple from parent environment.
-    os.chdir("/")
-    os.umask(0)
-    os.setsid()
-
-    # Do second fork.
-    try:
-        pid = os.fork()
-        if pid > 0:
-            sys.exit(0) # Exit second parent.
-    except OSError, e:
-        sys.stderr.write ("fork #2 failed: (%d) %s\n" % (e.errno, e.strerror))
-        sys.exit(1)
-
-    if prv_pidfile_name != "":
-        get_pidfile_lock()
-
-    # Now I am a daemon!
-
-    # Redirect standard file descriptors.
-    si = file(stdin, 'r')
-    so = file(stdout, 'a+')
-    se = file(stderr, 'a+', 0)
-    os.dup2(si.fileno(), sys.stdin.fileno())
-    os.dup2(so.fileno(), sys.stdout.fileno())
-    os.dup2(se.fileno(), sys.stderr.fileno())
-
-
-
-def ifelse(cond, t, f):
-    if cond: return t
-    else: return f
+##def ifelse(cond, t, f):
+##    if cond: return t
+##    else: return f
 
 def to_bool_str(s, default='0'):
     """ Convert an arbitrary 0/1/T/F/Y/N string to a normalized string 0/1."""
     if isinstance(s, str) and s:
         if s[0].lower() in ['1', 't', 'y']:
-            return '1'
+            return u'1'
         elif s[0].lower() in ['0', 'f', 'n']:
-            return '0'
+            return u'0'
 
     return default
 
@@ -175,16 +165,16 @@ def to_bool(s, default=False):
 
     return default
 
-def path_exists_safely(path):
-    """ Returns True if path exists, and points to a file with permissions at least as strict as 0755.
-        Credit: Contributed by Henrique M. Holschuh <hmh@debian.org>"""
-    try:
-        pathmode = os.stat(path)[stat.ST_MODE]
-        if pathmode & 0022 != 0:
-            return False
-    except (IOError,OSError):
-        return False
-    return True
+##def path_exists_safely(path):
+##    """ Returns True if path exists, and points to a file with permissions at least as strict as 0755.
+##        Credit: Contributed by Henrique M. Holschuh <hmh@debian.org>"""
+##    try:
+##        pathmode = os.stat(path)[stat.ST_MODE]
+##        if pathmode & 0022 != 0:
+##            return False
+##    except (IOError,OSError):
+##        return False
+##    return True
 
 
 def walkFiles(root, recurse=True, abs_paths=False, return_folders=False, pattern='*', path=None):
@@ -213,10 +203,10 @@ def walkFiles(root, recurse=True, abs_paths=False, return_folders=False, pattern
                         except ValueError:
                             yield fullname
 
-        if os.path.islink(fullname):
-            fullname = os.path.realpath(os.readlink(fullname))
+        #if os.path.islink(fullname):
+        #    fullname = os.path.realpath(os.readlink(fullname))
 
-        if recurse and os.path.isdir(fullname) or os.path.islink(fullname):
+        if recurse and os.path.isdir(fullname): # or os.path.islink(fullname):
             for f in walkFiles(fullname, recurse, abs_paths, return_folders, pattern, path):
                 yield f
 
@@ -268,12 +258,6 @@ class TextFormatter:
         else:
             return '\n'.join(complines)
 
-    def bold(text):
-        return ''.join(["\033[1m", text, "\033[0m"])
-
-    bold = staticmethod(bold)
-
-
 class Column:
 
     def __init__(self, width=78, alignment=TextFormatter.LEFT, margin=0):
@@ -317,6 +301,7 @@ class Column:
             return ' ' * (self.margin + self.width)
 
 
+            
 class Stack:
     def __init__(self):
         self.stack = []
@@ -326,12 +311,29 @@ class Stack:
 
     def push(self, value):
         self.stack.append(value)
-
+        
     def as_list(self):
         return self.stack
 
     def clear(self):
         self.stack = []
+        
+    def __len__(self):
+        return len(self.stack)
+        
+        
+        
+class Queue(Stack):
+    def __init__(self):
+        Stack.__init__(self)
+
+    def get(self):
+        return self.stack.pop(0)
+        
+    def put(self, value):
+        Stack.push(self, value)
+
+   
 
 
 # RingBuffer class
@@ -342,19 +344,19 @@ class RingBuffer:
     def __init__(self,size_max=50):
         self.max = size_max
         self.data = []
-    
+
     def append(self,x):
         """append an element at the end of the buffer"""
         self.data.append(x)
-        
+
         if len(self.data) == self.max:
             self.cur = 0
             self.__class__ = RingBufferFull
-            
+
     def replace(self, x):
         """replace the last element instead off appending"""
         self.data[-1] = x
-    
+
     def get(self):
         """ return a list of elements from the oldest to the newest"""
         return self.data
@@ -364,18 +366,18 @@ class RingBufferFull:
     def __init__(self,n):
         #raise "you should use RingBuffer"
         pass
-    
+
     def append(self,x):
         self.data[self.cur] = x
         self.cur = (self.cur+1) % self.max
-        
+
     def replace(self, x):
         # back up 1 position to previous location
         self.cur = (self.cur-1) % self.max
         self.data[self.cur] = x
         # setup for next item
         self.cur = (self.cur+1) % self.max
-    
+
     def get(self):
         return self.data[self.cur:] + self.data[:self.cur]
 
@@ -384,90 +386,10 @@ def sort_dict_by_value(d):
     items=d.items()
     backitems=[[v[1],v[0]] for v in items]
     backitems.sort()
-    return [backitems[i][1] for i in range(0,len(backitems))]
+    return [backitems[i][1] for i in range(0, len(backitems))]
 
-
-# Copied from Gentoo Portage output.py
-# Copyright 1998-2003 Daniel Robbins, Gentoo Technologies, Inc.
-# Distributed under the GNU Public License v2
-
-codes={}
-codes["reset"]="\x1b[0m"
-codes["bold"]="\x1b[01m"
-
-codes["teal"]="\x1b[36;06m"
-codes["turquoise"]="\x1b[36;01m"
-
-codes["fuscia"]="\x1b[35;01m"
-codes["purple"]="\x1b[35;06m"
-
-codes["blue"]="\x1b[34;01m"
-codes["darkblue"]="\x1b[34;06m"
-
-codes["green"]="\x1b[32;01m"
-codes["darkgreen"]="\x1b[32;06m"
-
-codes["yellow"]="\x1b[33;01m"
-codes["brown"]="\x1b[33;06m"
-
-codes["red"]="\x1b[31;01m"
-codes["darkred"]="\x1b[31;06m"
-
-
-def bold(text):
-    return codes["bold"]+text+codes["reset"]
-
-def white(text):
-    return bold(text)
-
-def teal(text):
-    return codes["teal"]+text+codes["reset"]
-
-def turquoise(text):
-    return codes["turquoise"]+text+codes["reset"]
-
-def darkteal(text):
-    return turquoise(text)
-
-def fuscia(text):
-    return codes["fuscia"]+text+codes["reset"]
-
-def purple(text):
-    return codes["purple"]+text+codes["reset"]
-
-def blue(text):
-    return codes["blue"]+text+codes["reset"]
-
-def darkblue(text):
-    return codes["darkblue"]+text+codes["reset"]
-
-def green(text):
-    return codes["green"]+text+codes["reset"]
-
-def darkgreen(text):
-    return codes["darkgreen"]+text+codes["reset"]
-
-def yellow(text):
-    return codes["yellow"]+text+codes["reset"]
-
-def brown(text):
-    return codes["brown"]+text+codes["reset"]
-
-def darkyellow(text):
-    return brown(text)
-
-def red(text):
-    return codes["red"]+text+codes["reset"]
-
-def darkred(text):
-    return codes["darkred"]+text+codes["reset"]
-
-
-# TODO: Use locale.format("%d", val, grouping=True) ???
 def commafy(val): 
-    return val < 0 and '-' + commafy(abs(val)) \
-        or val < 1000 and str(val) \
-        or '%s,%03d' % (commafy(val / 1000), (val % 1000))
+    return unicode(locale.format("%d", val, grouping=True))
 
 
 def format_bytes(s, show_bytes=False):
@@ -475,14 +397,19 @@ def format_bytes(s, show_bytes=False):
         return ''.join([commafy(s), ' B'])
     elif 1024 < s < 1048576:
         if show_bytes:
-            return ''.join([str(round(s/1024.0, 1)) , ' KB (',  commafy(s), ')'])
+            return ''.join([unicode(round(s/1024.0, 1)) , u' KB (',  commafy(s), ')'])
         else:
-            return ''.join([str(round(s/1024.0, 1)) , ' KB'])
+            return ''.join([unicode(round(s/1024.0, 1)) , u' KB'])
+    elif 1048576 < s < 1073741824:
+        if show_bytes:
+            return ''.join([unicode(round(s/1048576.0, 1)), u' MB (',  commafy(s), ')'])
+        else:
+            return ''.join([unicode(round(s/1048576.0, 1)), u' MB'])
     else:
         if show_bytes:
-            return ''.join([str(round(s/1048576.0, 1)), ' MB (',  commafy(s), ')'])
+            return ''.join([unicode(round(s/1073741824.0, 1)), u' GB (',  commafy(s), ')'])
         else:
-            return ''.join([str(round(s/1048576.0, 1)), ' MB'])
+            return ''.join([unicode(round(s/1073741824.0, 1)), u' GB'])
 
 
 
@@ -492,16 +419,19 @@ except AttributeError:
     def make_temp_file(suffix='', prefix='', dir='', text=False): # pre-2.3
         path = tempfile.mktemp(suffix)
         fd = os.open(path, os.O_RDWR|os.O_CREAT|os.O_EXCL, 0700)
-        #os.unlink( path ) # TODO... make this secure
         return ( os.fdopen( fd, 'w+b' ), path )
-        #return (fd, path)
 
-def log_title(program_name, version):
+def log_title(program_name, version, show_ver=True):
     log.info("")
-    log.info(bold("HP Linux Imaging and Printing System (ver. %s)" % prop.version))
-    log.info(bold("%s ver. %s" % (program_name, version)))
+
+    if show_ver:
+        log.info(log.bold("HP Linux Imaging and Printing System (ver. %s)" % prop.version))
+    else:    
+        log.info(log.bold("HP Linux Imaging and Printing System"))
+
+    log.info(log.bold("%s ver. %s" % (program_name, version)))
     log.info("")
-    log.info("Copyright (c) 2001-7 Hewlett-Packard Development Company, LP")
+    log.info("Copyright (c) 2001-8 Hewlett-Packard Development Company, LP")
     log.info("This software comes with ABSOLUTELY NO WARRANTY.")
     log.info("This is free software, and you are welcome to distribute it")
     log.info("under certain conditions. See COPYING file for more details.")
@@ -520,7 +450,7 @@ def which(command, return_full_path=False):
     for p in path:
         try:
             files = os.listdir(p)
-        except:
+        except OSError:
             continue
         else:
             if command in files:
@@ -539,89 +469,90 @@ def which(command, return_full_path=False):
 class UserSettings(object):
     def __init__(self):
         self.load()
-    
+
     def loadDefaults(self):
         # Print
+        self.cmd_print = ''
         path = which('hp-print')
-    
+
         if len(path) > 0:
             self.cmd_print = 'hp-print -p%PRINTER%'
         else:
             path = which('kprinter')
-    
+
             if len(path) > 0:
                 self.cmd_print = 'kprinter -P%PRINTER% --system cups'
             else:
                 path = which('gtklp')
-    
+
                 if len(path) > 0:
                     self.cmd_print = 'gtklp -P%PRINTER%'
-    
+
                 else:
                     path = which('xpp')
-    
+
                     if len(path) > 0:
                         self.cmd_print = 'xpp -P%PRINTER%'
-    
+
         # Scan
+        self.cmd_scan = ''
         path = which('xsane')
-    
+
         if len(path) > 0:
             self.cmd_scan = 'xsane -V %SANE_URI%'
         else:
             path = which('kooka')
-    
-            if len(path)>0:
-                #cmd_scan = 'kooka -d "%SANE_URI%"'
+
+            if len(path) > 0:
                 self.cmd_scan = 'kooka'
-    
+
             else:
                 path = which('xscanimage')
-    
-                if len(path)>0:
+
+                if len(path) > 0:
                     self.cmd_scan = 'xscanimage'
-    
+
         # Photo Card
         path = which('hp-unload')
-    
+
         if len(path):
             self.cmd_pcard = 'hp-unload -d %DEVICE_URI%'
-    
+
         else:
             self.cmd_pcard = 'python %HOME%/unload.py -d %DEVICE_URI%'
-    
+
         # Copy
         path = which('hp-makecopies')
-    
+
         if len(path):
             self.cmd_copy = 'hp-makecopies -d %DEVICE_URI%'
-    
+
         else:
             self.cmd_copy = 'python %HOME%/makecopies.py -d %DEVICE_URI%'
-    
+
         # Fax
         path = which('hp-sendfax')
-    
+
         if len(path):
             self.cmd_fax = 'hp-sendfax -d %FAX_URI%'
-    
+
         else:
             self.cmd_fax = 'python %HOME%/sendfax.py -d %FAX_URI%'
-    
+
         # Fax Address Book
         path = which('hp-fab')
-    
+
         if len(path):
             self.cmd_fab = 'hp-fab'
-    
+
         else:
             self.cmd_fab = 'python %HOME%/fab.py'    
-    
+
     def load(self):
         self.loadDefaults()
-        
+
         log.debug("Loading user settings...")
-        
+
         self.email_alerts = to_bool(user_cfg.alerts.email_alerts, False)
         self.email_to_addresses = user_cfg.alerts.email_to_addresses
         self.email_from_address = user_cfg.alerts.email_from_address
@@ -639,80 +570,80 @@ class UserSettings(object):
 
         self.cmd_print = user_cfg.commands.prnt or self.cmd_print
         self.cmd_print_int = to_bool(user_cfg.commands.prnt_int, True)
-        
+
         self.cmd_scan = user_cfg.commands.scan or self.cmd_scan
         self.cmd_scan_int = to_bool(user_cfg.commands.scan_int, False)
-        
+
         self.cmd_pcard = user_cfg.commands.pcard or self.cmd_pcard
         self.cmd_pcard_int = to_bool(user_cfg.commands.pcard_int, True)
-        
+
         self.cmd_copy = user_cfg.commands.cpy or self.cmd_copy
         self.cmd_copy_int = to_bool(user_cfg.commands.cpy_int, True)
-        
+
         self.cmd_fax = user_cfg.commands.fax or self.cmd_fax
         self.cmd_fax_int = to_bool(user_cfg.commands.fax_int, True)
-        
+
         self.cmd_fab = user_cfg.commands.fab or self.cmd_fab
         self.cmd_fab_int = to_bool(user_cfg.commands.fab_int, False)
-        
+
         self.debug()
-    
+
     def debug(self):
         log.debug("Print command: %s" % self.cmd_print)
         log.debug("Use Internal print command: %s" % self.cmd_print_int)
-        
+
         log.debug("PCard command: %s" % self.cmd_pcard)
         log.debug("Use internal PCard command: %s" % self.cmd_pcard_int)
-        
+
         log.debug("Fax command: %s" % self.cmd_fax)
         log.debug("Use internal fax command: %s" % self.cmd_fax_int)
-        
+
         log.debug("FAB command: %s" % self.cmd_fab)
         log.debug("Use internal FAB command: %s" % self.cmd_fab_int)
-        
+
         log.debug("Copy command: %s " % self.cmd_copy)
         log.debug("Use internal copy command: %s " % self.cmd_copy_int)
-        
+
         log.debug("Scan command: %s" % self.cmd_scan)
         log.debug("Use internal scan command: %s" % self.cmd_scan_int)
-        
+
         log.debug("Email alerts: %s" % self.email_alerts)
         log.debug("Email to address(es): %s" % self.email_to_addresses)
         log.debug("Email from address: %s" % self.email_from_address)
         log.debug("Auto refresh: %s" % self.auto_refresh)
         log.debug("Auto refresh rate: %s" % self.auto_refresh_rate)
         log.debug("Auto refresh type: %s" % self.auto_refresh_type)        
-    
+
     def save(self):
         log.debug("Saving user settings...")
-        
+
         user_cfg.commands.prnt = self.cmd_print
         user_cfg.commands.prnt_int = self.cmd_print_int
-        
+
         user_cfg.commands.pcard = self.cmd_pcard
         user_cfg.commands.pcard_int = self.cmd_pcard_int
-        
+
         user_cfg.commands.fax = self.cmd_fax
         user_cfg.commands.fax_int = self.cmd_fax_int
-        
+
         user_cfg.commands.scan = self.cmd_scan
         user_cfg.commands.scan_int = self.cmd_scan_int
-        
+
         user_cfg.commands.cpy = self.cmd_copy
         user_cfg.commands.cpy_int = self.cmd_copy_int
-        
+
         user_cfg.alerts.email_to_addresses = self.email_to_addresses
         user_cfg.alerts.email_from_address = self.email_from_address
         user_cfg.alerts.email_alerts = self.email_alerts
-        
+
         user_cfg.refresh.enable = self.auto_refresh
         user_cfg.refresh.rate = self.auto_refresh_rate
         user_cfg.refresh.type = self.auto_refresh_type
-        
-        self.debug()
-        
 
-          
+        self.debug()
+
+
+
 def no_qt_message_gtk():
     try:
         import gtk
@@ -724,7 +655,23 @@ def no_qt_message_gtk():
         dialog.destroy()
 
     except ImportError:
-        pass
+        log.error("PyQt not installed. GUI not available. Please check that the PyQt package is installed. Exiting.")
+
+
+def canEnterGUIMode():
+    if not prop.gui_build:
+        log.warn("GUI mode disabled in build.")
+        return False
+
+    elif not os.getenv('DISPLAY'):
+        log.warn("No display found.")
+        return False
+
+    elif not checkPyQtImport():
+        log.warn("Qt/PyQt initialization failed.")
+        return False
+
+    return True
 
 def checkPyQtImport():
     # PyQt
@@ -748,7 +695,7 @@ def checkPyQtImport():
     #check version of PyQt
     try:
         pyqtVersion = qt.PYQT_VERSION_STR
-    except:
+    except AttributeError:
         pyqtVersion = qt.PYQT_VERSION
 
     while pyqtVersion.count('.') < 2:
@@ -775,58 +722,6 @@ def checkPyQtImport():
 
     return True
 
-
-def loadTranslators(app, user_config):
-    #from qt import *
-    import qt
-    loc = None
-
-    if os.path.exists(user_config):
-        # user_config contains executables we will run, so we
-        # must make sure it is a safe file, and refuse to run
-        # otherwise.
-        if not path_exists_safely(user_config):
-            log.warning("File %s has insecure permissions! File ignored." % user_config)
-        else:
-            config = ConfigParser.ConfigParser()
-            config.read(user_config)
-
-            if config.has_section("ui"):
-                loc = config.get("ui", "loc")
-
-                if not loc:
-                    loc = None
-
-    if loc is not None:
-
-        if loc.lower() == 'system':
-            loc = str(qt.QTextCodec.locale())
-
-        if loc.lower() != 'c':
-
-            log.debug("Trying to load .qm file for %s locale." % loc)
-
-            dirs = [prop.home_dir, prop.data_dir, prop.i18n_dir]
-
-            trans = qt.QTranslator(None)
-
-            for dir in dirs:
-                qm_file = 'hplip_%s' % loc
-                loaded = trans.load(qm_file, dir)
-
-                if loaded:
-                    app.installTranslator(trans)
-                    break
-        else:
-            loc = None
-
-    if loc is None:
-        log.debug("Using default 'C' locale")
-    else:
-        log.debug("Using locale: %s" % loc)
-
-    return loc
-
 try:
     from string import Template # will fail in Python <= 2.3
 except ImportError:
@@ -883,7 +778,6 @@ except ImportError:
             self.template = template
 
         # Search for $$, $identifier, ${identifier}, and any bare $'s
-
         def _invalid(self, mo):
             i = mo.start('invalid')
             lines = self.template[:i].splitlines(True)
@@ -957,7 +851,19 @@ except ImportError:
 
 
 
-cat = lambda _ : Template(_).substitute(sys._getframe(1).f_globals, **sys._getframe(1).f_locals)
+#cat = lambda _ : Template(_).substitute(sys._getframe(1).f_globals, **sys._getframe(1).f_locals)
+
+def cat(s):
+    globals = sys._getframe(1).f_globals.copy()
+    if 'self' in globals:
+        del globals['self']
+    
+    locals = sys._getframe(1).f_locals.copy()
+    if 'self' in locals:
+        del locals['self']
+        
+    return Template(s).substitute(sys._getframe(1).f_globals, **locals)
+    
 identity = string.maketrans('','')
 unprintable = identity.translate(identity, string.printable)
 
@@ -975,18 +881,35 @@ def all(S,f=lambda x:x):
         if not f(x): return False
     return True
 
-def openURL(url):
-    browsers = ['firefox', 'mozilla', 'konqueror', 'galeon', 'skipstone'] # in preferred order
-    browser_opt = {'firefox': '-new-window', 'mozilla' : '', 'konqueror': '', 'galeon': '-w', 'skipstone': ''}
+BROWSERS = ['firefox', 'mozilla', 'konqueror', 'galeon', 'skipstone'] # in preferred order
+BROWSER_OPTS = {'firefox': '-new-window', 'mozilla' : '', 'konqueror': '', 'galeon': '-w', 'skipstone': ''}
 
-    for b in browsers:
-        if which(b):
-            cmd = """%s %s "%s" &""" % (b, browser_opt[b], url)
-            log.debug(cmd)
-            os.system(cmd)
-            break
+def find_browser():
+    if platform_avail and platform.system() == 'Darwin':
+        return "open"
     else:
-        log.warn("Unable to open URL: %s" % url)
+        for b in BROWSERS:
+            if which(b):
+                return b
+        else:
+            return None
+    
+def openURL(url):
+    if platform_avail and platform.system() == 'Darwin':
+        cmd = 'open "%s"' % url
+        log.debug(cmd)
+        os.system(cmd)
+    else:
+        for b in BROWSERS:
+            bb = which(b)
+            if bb:
+                bb = os.path.join(bb, b)
+                cmd = """%s %s "%s" &""" % (bb, BROWSER_OPTS[b], url)
+                log.debug(cmd)
+                os.system(cmd)
+                break
+        else:
+            log.warn("Unable to open URL: %s" % url)
 
 
 def uniqueList(input):
@@ -1073,11 +996,11 @@ class XMLToDictParser:
 
 
  # ------------------------- Usage Help
-
 USAGE_OPTIONS = ("[OPTIONS]", "", "heading", False)
 USAGE_LOGGING1 = ("Set the logging level:", "-l<level> or --logging=<level>", 'option', False)
 USAGE_LOGGING2 = ("", "<level>: none, info\*, error, warn, debug (\*default)", "option", False)
 USAGE_LOGGING3 = ("Run in debug mode:", "-g (same as option: -ldebug)", "option", False)
+USAGE_LOGGING_PLAIN = ("Output plain text only:", "-t", "option", False)
 USAGE_ARGS = ("[PRINTER|DEVICE-URI] (See Notes)", "", "heading", False)
 USAGE_DEVICE = ("To specify a device-URI:", "-d<device-uri> or --device=<device-uri>", "option", False)
 USAGE_PRINTER = ("To specify a CUPS printer:", "-p<printer> or --printer=<printer>", "option", False)
@@ -1090,6 +1013,8 @@ USAGE_NOTES = ("Notes:", "", "heading", False)
 USAGE_STD_NOTES1 = ("1. If device or printer is not specified, the local device bus is probed and the program enters interactive mode.", "", "note", False)
 USAGE_STD_NOTES2 = ("2. If -p\* is specified, the default CUPS printer will be used.", "", "note", False)
 USAGE_SEEALSO = ("See Also:", "", "heading", False)
+USAGE_LANGUAGE = ("Set the language:", "-q <lang> or --lang=<lang>. Use -q? or --lang=? to see a list of available language codes.", "option", False)
+USAGE_LANGUAGE2 = ("Set the language:", "--lang=<lang>. Use --lang=? to see a list of available language codes.", "option", False)
 
 def ttysize():
     ln1 = commands.getoutput('stty -a').splitlines()[0]
@@ -1139,7 +1064,7 @@ def format_text(text_list, typ='text', title='', crumb='', version=''):
             text2 = text2.replace("\\", "")
 
             if format == 'summary':
-                log.info(bold(text1))
+                log.info(log.bold(text1))
                 log.info("")
 
             elif format in ('para', 'name', 'seealso'):
@@ -1149,7 +1074,7 @@ def format_text(text_list, typ='text', title='', crumb='', version=''):
                     log.info("")
 
             elif format in ('heading', 'header'):
-                log.info(bold(text1))
+                log.info(log.bold(text1))
 
             elif format in ('option', 'example'):
                 log.info(formatter.compose((text1, text2), trailing_space))
@@ -1302,16 +1227,18 @@ def xreverse(s):
 def xstrip(s, chars=' '):
     return xreverse(xlstrip(xreverse(xlstrip(s, chars)), chars))
 
-
-
 def getBitness():
-    try:
-        import platform
-    except ImportError:
-        return struct.calcsize("P") << 3
-    else:
+    if platform_avail:
         return int(platform.architecture()[0][:-3])
+    else:
+        return struct.calcsize("P") << 3
 
+def getProcessor():
+    if platform_avail:
+        return platform.machine().replace(' ', '_').lower() # i386, i686, power_macintosh, etc.
+    else:
+        return "i686" # TODO: Need a fix here
+    
 
 BIG_ENDIAN = 0
 LITTLE_ENDIAN = 1
@@ -1325,6 +1252,7 @@ def getEndian():
 
 def get_password():
     return getpass.getpass("Enter password: ")
+    
 
 def run(cmd, log_output=True, password_func=get_password, timeout=1):
     output = cStringIO.StringIO()
@@ -1410,10 +1338,10 @@ def expand_range(ns): # ns -> string repr. of numeric range, e.g. "1-4, 7, 9-12"
 def collapse_range(x): # x --> sorted list of ints
     """ Convert a list of integers into a string
         range representation: 
-        [1,2,3,4,7,9,10,11,12] --> "1-4, 7, 9-12"
+        [1,2,3,4,7,9,10,11,12] --> "1-4,7,9-12"
     """
     if not x:
-        return ""
+        return ''
 
     s, c, r = [str(x[0])], x[0], False
 
@@ -1422,10 +1350,10 @@ def collapse_range(x): # x --> sorted list of ints
             r = True
         else:
             if r:
-                s.append('-%s, %s' % (c,i))
+                s.append('-%s,%s' % (c,i))
                 r = False
             else:
-                s.append(', %s' % i)
+                s.append(',%s' % i)
 
         c = i
 
@@ -1434,3 +1362,121 @@ def collapse_range(x): # x --> sorted list of ints
 
     return ''.join(s)
 
+def createSequencedFilename(basename, ext, dir=None, digits=3):
+    if dir is None:
+        dir = os.getcwd()
+
+    m = 0
+    for f in walkFiles(dir, recurse=False, abs_paths=False, return_folders=False, pattern='*', path=None):
+        r, e = os.path.splitext(f)
+
+        if r.startswith(basename) and ext == e:
+            try:
+                i = int(r[len(basename):])
+            except ValueError:
+                continue
+            else:
+                m = max(m, i)
+
+    return os.path.join(dir, "%s%0*d%s" % (basename, digits, m+1, ext))
+
+
+def validate_language(lang, default='en_US'):
+    if lang is None:
+        loc, encoder = locale.getdefaultlocale()
+    else:
+        lang = lang.lower().strip()
+        for loc, ll in supported_locales.items():
+            if lang in ll:
+                break
+        else:
+            loc = user_cfg.ui.get("loc", "en_US")
+            log.warn("Unknown lang/locale. Using default of %s." % loc)
+
+    return loc
+
+   
+def gen_random_uuid():
+    try:
+        import uuid # requires Python 2.5+
+        return str(uuid.uuid4())
+        
+    except ImportError:
+        uuidgen = which("uuidgen")
+        if uuidgen:
+            uuidgen = os.path.join(uuidgen, "uuidgen")
+            return commands.getoutput(uuidgen)
+        else:
+            return ''
+            
+            
+class RestTableFormatter(object):
+    def __init__(self, header=None):
+        self.header = header # tuple of strings
+        self.rows = [] # list of tuples
+
+    def add(self, row_data): # tuple of strings
+        self.rows.append(row_data)
+
+    def output(self, w):
+        if self.rows:
+            num_cols = len(self.rows[0])
+            for r in self.rows:
+                if len(r) != num_cols:
+                    log.error("Invalid number of items in row: %s" % r)
+                    return
+
+            if len(self.header) != num_cols:
+                log.error("Invalid number of items in header.")
+
+            col_widths = []
+            for x, c in enumerate(self.header):
+                max_width = len(c)
+                for r in self.rows:
+                    max_width = max(max_width, len(r[x]))
+
+                col_widths.append(max_width+2)
+
+            x = '+'
+            for c in col_widths:
+                x = ''.join([x, '-' * (c+2), '+'])
+
+            x = ''.join([x, '\n'])
+            w.write(x)
+
+            # header
+            if self.header:
+                x = '|'
+                for i, c in enumerate(col_widths):
+                    x = ''.join([x, ' ', self.header[i], ' ' * (c+1-len(self.header[i])), '|'])
+
+                x = ''.join([x, '\n'])
+                w.write(x)
+
+                x = '+'
+                for c in col_widths:
+                    x = ''.join([x, '=' * (c+2), '+'])
+
+                x = ''.join([x, '\n'])
+                w.write(x)
+
+            # data rows
+            for j, r in enumerate(self.rows):
+                x = '|'
+                for i, c in enumerate(col_widths):
+                    x = ''.join([x, ' ', self.rows[j][i], ' ' * (c+1-len(self.rows[j][i])), '|'])
+
+                x = ''.join([x, '\n'])
+                w.write(x)
+
+                x = '+'
+                for c in col_widths:
+                    x = ''.join([x, '-' * (c+2), '+'])
+
+                x = ''.join([x, '\n'])
+                w.write(x)
+
+        else:
+            log.error("No data rows")
+
+    
