@@ -28,7 +28,7 @@ from cStringIO import StringIO
 # Local
 from base.g import *
 from base.codes import *
-from base import device, utils, status, pml, msg
+from base import device, utils, status, pml
 
 # Event queue values (UI ==> Copy thread)
 COPY_CANCELED = 1
@@ -44,11 +44,11 @@ STATUS_ERROR = 5
 
 # PML Copier Only
 class PMLCopyDevice(device.Device):
-    def __init__(self, device_uri=None, printer_name=None,
-                 hpssd_sock=None, hpiod_sock=None, callback=None):
+    def __init__(self, device_uri=None, printer_name=None, 
+                 service=None, callback=None):
 
         device.Device.__init__(self, device_uri, printer_name,
-                               hpssd_sock, hpiod_sock, callback)
+                               service, callback)
 
         self.copy_thread = None
 
@@ -115,12 +115,15 @@ class PMLCopyThread(threading.Thread):
         state = STATE_SET_TOKEN
 
         while state != STATE_DONE: # ------------------------- Copier Thread
-            if self.check_for_cancel():
-                state = STATE_ABORTED
+            # revisit - Checking cancel and setting state here means
+            # every state can unconditionally transition to STATE_ABORTED.
+            # This has not been verified.
+            # if self.check_for_cancel():
+                # state = STATE_ABORTED
 
             if state == STATE_ABORTED:
                 log.debug("%s State: Aborted" % ("*"*20))
-                self.write_queue(STATUS_ERROR)
+                self.write_queue(STATUS_DONE) # This was STATUS_ERROR.
                 state = STATE_RESET_TOKEN
 
             if state == STATE_ERROR:
@@ -347,6 +350,8 @@ class PMLCopyThread(threading.Thread):
                 except Error:
                     log.error("Unable to release copier token.")
 
+                self.dev.close() # Close the device.
+                
                 state = STATE_DONE
 
 
