@@ -54,7 +54,11 @@ DJGenericVIP::DJGenericVIP (SystemServices* pSS, BOOL proto)
     {
         bCheckForCancelButton = TRUE;
         constructor_error = VerifyPenInfo ();
-        CERRCHECK;
+        if (constructor_error != NO_ERROR)
+        {
+            constructor_error = NO_ERROR;
+            ePen = BOTH_PENS;
+        }
     }
     else
         ePen = BOTH_PENS;
@@ -213,7 +217,7 @@ PAPER_SIZE DJGenericVIP::MandatoryPaperSize ()
  *      100 - 13 X 19 size
  */
 
-    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
+    if (pSS->IOMode.bDevID && (pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -251,7 +255,7 @@ BOOL DJGenericVIP::FullBleedCapable (PAPER_SIZE ps, FullbleedType  *fbType, floa
     char    *pStr;
     sDevIdStr[0] = 0;
 
-    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
+    if (pSS->IOMode.bDevID && (pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -608,6 +612,36 @@ DRIVER_ERROR DJGenericVIP::SendPerPageHeader (BOOL bLastPage)
 }
 #endif // APDK_LINUX
 
+Header *DJGenericVIP::SelectHeader (PrintContext *pc)
+{
+    return new HeaderDJGenericVIP (this, pc);
+}
+
+HeaderDJGenericVIP::HeaderDJGenericVIP (Printer *p, PrintContext *pc) : HeaderDJ990 (p, pc)
+{
+    m_uiCAPy = 0;
+}
+
+DRIVER_ERROR HeaderDJGenericVIP::SendCAPy (unsigned int iAbsY)
+{
+    char    str[16];
+    DRIVER_ERROR err = NO_ERROR;
+    if (m_uiCAPy == 0)
+    {
+        sprintf (str, "\x1B*p%dY", iAbsY);
+        err = thePrinter->Send ((const BYTE *) str, strlen (str));
+        m_uiCAPy = iAbsY;
+    }
+    return err;
+}
+
+DRIVER_ERROR HeaderDJGenericVIP::FormFeed ()
+{
+    BYTE FF = 12;
+    m_uiCAPy = 0;
+    return thePrinter->Send ((const BYTE *) &FF, 1);
+}
+
 APDK_END_NAMESPACE
 
 
