@@ -380,6 +380,14 @@ class LJJetReadyProxy;
 class LJFastRasterProxy;
 #endif
 
+#ifdef APDK_LJZJS_MONO
+class LJZjsMonoProxy;
+#endif
+
+#ifdef APDK_LJM1005
+class LJM1005Proxy;
+#endif
+
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
 class PSP100Proxy;
 class PSP470Proxy;
@@ -522,6 +530,14 @@ public:
     static LJFastRasterProxy s_LJFastRasterProxy;
 #endif
 
+#ifdef APDK_LJZJS_MONO
+    static LJZjsMonoProxy s_LJZjsMonoProxy;
+#endif
+
+#ifdef APDK_LJM1005
+    static LJM1005Proxy s_LJM1005Proxy;
+#endif
+
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
     static PSP100Proxy s_PSP100Proxy;
     static PSP470Proxy s_PSP470Proxy;
@@ -973,6 +989,18 @@ public:
     void SetMediaSource(MediaSource msource);
 }; //HeaderDJ990
 
+//ClassName
+/*
+******************************************************************************/
+class HeaderDJGenericVIP : public HeaderDJ990
+{
+public:
+    HeaderDJGenericVIP (Printer *p, PrintContext *pc);
+protected:
+    unsigned int m_uiCAPy;
+    DRIVER_ERROR SendCAPy (unsigned int iAbsY);
+    DRIVER_ERROR FormFeed ();
+}; // HeaderDJGenericVIP
 
 //ClassName
 /*
@@ -1106,6 +1134,21 @@ protected:
 	int			 FrPaperToMediaSize(PAPER_SIZE ps);
 }; //HeaderLJFastRaster
 
+//ClassName
+/*
+******************************************************************************/
+class HeaderLJZjs : public Header
+{
+    friend class LJZjs;
+public:
+    HeaderLJZjs (Printer *p, PrintContext *pc);
+    virtual DRIVER_ERROR Send ();
+    virtual DRIVER_ERROR FormFeed ();
+protected:
+    DRIVER_ERROR EndJob ();
+    DRIVER_ERROR SendCAPy (unsigned int iAbsY);
+}; // HeaderLJZjs
+
 class RasterSender : public Processor
 {
 friend class Header;
