@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2006 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2008 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,16 +19,20 @@
 # Author: Don Welch
 #
 
-import sys
+# Std Lib
+
+# Local
+from base.g import *
+from ui_utils import load_pixmap
+
+# Qt
 from qt import *
 from colorcalform2_base import ColorCalForm2_base
-from base.g import *
-import os.path
 
 class ColorCalForm2(ColorCalForm2_base):
     def __init__(self,parent = None,name = None,modal = 0,fl = 0):
         ColorCalForm2_base.__init__(self,parent,name,modal,fl)
-        self.Icon.setPixmap(QPixmap(os.path.join(prop.image_dir, 'color-adj.png')))
+        self.Icon.setPixmap(load_pixmap('color_adj', 'other'))
         self.value = 1
 
     def SpinBox_valueChanged(self,a0):
