@@ -23,6 +23,7 @@
 from base.g import *
 from base import utils, magic
 from prnt import cups
+from ui_utils import load_pixmap
 
 # Qt
 from qt import *
@@ -38,29 +39,31 @@ class RangeValidator(QValidator):
         QValidator.__init__(self, parent, name)
 
     def validate(self, input, pos):
-        for x in str(input)[pos-1:]:
-            if x not in '0123456789,- ':
+        for x in unicode(input)[pos-1:]:
+            if x not in u'0123456789,- ':
                 return QValidator.Invalid, pos
 
         return QValidator.Acceptable, pos
         
 
 class FileListViewItem(QListViewItem):
-    def __init__(self, parent, order, filename, mime_type_desc, path):
-        QListViewItem.__init__(self, parent, order, filename, mime_type_desc, path)
+    def __init__(self, parent, index, filename, mime_type_desc, path):
+        QListViewItem.__init__(self, parent, str(index+1), filename, mime_type_desc, path)
         self.path = path
+        self.index = index
 
 
 class ScrollPrintView(ScrollView):
-    def __init__(self, toolbox_hosted=True, parent = None, form=None, name = None,fl = 0):
-        ScrollView.__init__(self,parent,name,fl)
-
+    def __init__(self, service, toolbox_hosted=True, parent=None, form=None, name=None, fl=0):
+        ScrollView.__init__(self, service, parent, name, fl)
+        
         self.toolbox_hosted = toolbox_hosted
         self.form = form
         self.file_list = []
         self.pages_button_group = 0
-        self.prev_selected_file_path = ''
-
+        #self.prev_selected_file_path = ''
+        self.prev_selected_file_index = 0
+        
         self.allowable_mime_types = cups.getAllowableMIMETypes()
         self.allowable_mime_types.append("application/x-python")
 
@@ -100,34 +103,38 @@ class ScrollPrintView(ScrollView):
     def fillControls(self):
         ScrollView.fillControls(self)
 
-        self.addPrinterFaxList()
-        self.addGroupHeading("files_to_print", self.__tr("File(s) to Print"))
-        self.addFileList()
-        self.addGroupHeading("options", self.__tr("Print Options"))
-        self.addCopies()
-        self.addPageRange()
-        self.addPageSet()
-        self.addGroupHeading("space1", "")
-
-        if self.toolbox_hosted:
-            s = self.__tr("<< Functions")
+        if self.addPrinterFaxList():
+            self.addGroupHeading("files_to_print", self.__tr("File(s) to Print"))
+            self.addFileList()
+            self.addGroupHeading("options", self.__tr("Print Options"))
+            self.addCopies()
+            self.addPageRange()
+            self.addPageSet()
+            self.addGroupHeading("space1", "")
+    
+            if self.toolbox_hosted:
+                s = self.__tr("<< Functions")
+            else:
+                s = self.__tr("Close")
+    
+            self.printButton = self.addActionButton("bottom_nav", self.__tr("Print File(s)"), 
+                                    self.printButton_clicked, 'print.png', 'print-disabled.png', 
+                                    s, self.funcButton_clicked)
+    
+            self.printButton.setEnabled(False)
+            self.maximizeControl()
+        
         else:
-            s = self.__tr("Close")
-
-        self.printButton = self.addActionButton("bottom_nav", self.__tr("Print File(s)"), 
-                                self.printButton_clicked, 'print.png', 'print-disabled.png', 
-                                s, self.funcButton_clicked)
-
-        self.printButton.setEnabled(False)
-
-        self.maximizeControl()
+            QApplication.restoreOverrideCursor()
+            self.form.FailureUI("<b>Print is disabled.</b><p>No CUPS print queue found for this device.")
+            self.funcButton_clicked()
+            
         
     def onUpdate(self, cur_device=None):
         log.debug("ScrollPrintView.onUpdate()")
         self.updateFileList()
         
     def onDeviceChange(self, cur_device=None):
-        #print "ScrollPrintView.onDeviceChange()"
         self.file_list = []
         ScrollView.onDeviceChange(self, cur_device)
 
@@ -136,12 +143,12 @@ class ScrollPrintView(ScrollView):
         layout37 = QGridLayout(widget,1,1,5,10,"layout37")
 
         self.addFilePushButton = PixmapLabelButton(widget, 
-            "list-add.png", "list-add-disabled.png")
+            "list_add.png", "list_add-disabled.png")
 
         layout37.addWidget(self.addFilePushButton,2,0)
 
         self.removeFilePushButton = PixmapLabelButton(widget, 
-            "list-remove.png", "list-remove-disabled.png")
+            "list_remove.png", "list_remove-disabled.png")
 
         layout37.addWidget(self.removeFilePushButton,2,1)
 
@@ -159,8 +166,6 @@ class ScrollPrintView(ScrollView):
             None, name='addFilePushButton')
 
         layout37.addWidget(self.showTypesPushButton,2,5)
-
-
         self.fileListView = QListView(widget,"fileListView")
         self.fileListView.addColumn(self.__tr("Order"))
         self.fileListView.addColumn(self.__tr("Name"))
@@ -187,6 +192,8 @@ class ScrollPrintView(ScrollView):
         self.moveFileUpPushButton.setText(self.__tr("Move Up"))
 
         self.removeFilePushButton.setEnabled(False)
+        self.moveFileDownPushButton.setEnabled(False)
+        self.moveFileUpPushButton.setEnabled(False)
 
         self.connect(self.addFilePushButton, SIGNAL("clicked()"), self.addFile_clicked)
         self.connect(self.removeFilePushButton, SIGNAL("clicked()"), self.removeFile_clicked)
@@ -202,7 +209,7 @@ class ScrollPrintView(ScrollView):
 
     def fileListView_selectionChanged(self, i):
         try:
-            self.prev_selected_file_path = i.path
+            self.prev_selected_file_index = i.index
         except AttributeError:
             pass
         else:
@@ -216,15 +223,14 @@ class ScrollPrintView(ScrollView):
             self.moveFileDownPushButton.setEnabled(file_count > 1 and selected_item is not last_item)
             self.moveFileUpPushButton.setEnabled(file_count > 1 and selected_item is not flv.firstChild())        
 
-
     def fileListView_rightButtonClicked(self, item, pos, col):
         popup = QPopupMenu(self)
-        popup.insertItem(QIconSet(QPixmap(os.path.join(prop.image_dir, 'list-add.png'))),
+        popup.insertItem(QIconSet(load_pixmap('list_add', '16x16')),
             self.__tr("Add File..."), self.addFile_clicked)
 
         if item is not None:
-            popup.insertItem(QIconSet(QPixmap(os.path.join(prop.image_dir,
-                'list-remove.png'))), self.__tr("Remove File"), self.removeFile_clicked)
+            popup.insertItem(QIconSet(load_pixmap('list_remove', '16x16')),
+                self.__tr("Remove File"), self.removeFile_clicked)
                 
             if self.fileListView.childCount() > 1:
                 last_item = self.fileListView.firstChild()
@@ -232,48 +238,50 @@ class ScrollPrintView(ScrollView):
                     last_item = last_item.nextSibling()
                     
                 if item is not self.fileListView.firstChild():
-                    popup.insertItem(QIconSet(QPixmap(os.path.join(prop.image_dir,
-                        'up.png'))), self.__tr("Move Up"), self.moveFileUp_clicked)
+                    popup.insertItem(QIconSet(load_pixmap('up', '16x16')),
+                        self.__tr("Move Up"), self.moveFileUp_clicked)
     
                 if item is not last_item:
-                    popup.insertItem(QIconSet(QPixmap(os.path.join(prop.image_dir,
-                        'down.png'))), self.__tr("Move Down"), self.moveFileDown_clicked)
+                    popup.insertItem(QIconSet(load_pixmap('down', '16x16')),
+                        self.__tr("Move Down"), self.moveFileDown_clicked)
 
         popup.insertSeparator(-1)
-        popup.insertItem(QIconSet(QPixmap(os.path.join(prop.image_dir,
-                'mimetypes.png'))), self.__tr("Show File Types..."), self.showFileTypes_clicked)
+        popup.insertItem(QIconSet(load_pixmap('mimetypes', '16x16')),
+            self.__tr("Show File Types..."), self.showFileTypes_clicked)
 
         popup.popup(pos)
 
-
     def moveFileUp_clicked(self):
+        log.debug("Move file up")
         try:
-            path = self.fileListView.selectedItem().path
+            item = self.fileListView.selectedItem()
+            path, index = item.path, item.index
         except AttributeError:
             return
         else:
-            for i in range(1, len(self.file_list)):
-                if self.file_list[i][0] == path:
-                    self.file_list[i-1],self.file_list[i] = self.file_list[i], self.file_list[i-1]
-
+            new_pos = index-1
+            self.file_list[new_pos], self.file_list[index] = self.file_list[index], self.file_list[new_pos]
+            item.index = new_pos
+            self.prev_selected_file_index = new_pos
             self.updateFileList()
 
     def moveFileDown_clicked(self):
+        log.debug("Move file down")
         try:
-            path = self.fileListView.selectedItem().path
+            item = self.fileListView.selectedItem()
+            path, index = item.path, item.index
         except AttributeError:
             return
         else:
-            for i in range(len(self.file_list) - 2, -1, -1):
-                if self.file_list[i][0] == path:
-                    self.file_list[i], self.file_list[i+1] = self.file_list[i+1], self.file_list[i] 
-
+            new_pos = index+1
+            self.file_list[index], self.file_list[new_pos] = self.file_list[new_pos], self.file_list[index] 
+            item.index = new_pos
+            self.prev_selected_file_index = new_pos
             self.updateFileList()
 
-
     def addFile(self, path):
         path = os.path.realpath(path)
-        if os.path.exists(path):
+        if os.path.exists(path) and os.access(path, os.R_OK):
             mime_type = magic.mime_type(path)
             mime_type_desc = mime_type
             log.debug(mime_type)
@@ -281,13 +289,12 @@ class ScrollPrintView(ScrollView):
             try:
                 mime_type_desc = self.MIME_TYPES_DESC[mime_type][0]
             except KeyError:
-                self.form.WarningUI(self.__tr("<b>You are trying to add a file that cannot be directly printed with this utility.</b><p>To print this file, use the print command in the application that created it."))
+                self.form.FailureUI(self.__tr("<b>You are trying to add a file that cannot be directly printed with this utility.</b><p>To print this file, use the print command in the application that created it."))
             else:
                 log.debug("Adding file %s (%s,%s)" % (path, mime_type, mime_type_desc))
                 self.file_list.append((path, mime_type, mime_type_desc))
-                self.prev_selected_file_path = path
         else:
-            self.form.FailureUI(self.__tr("<b>Unable to add file '%s' to file list.</b><p>Check the file name and try again." % path))
+            self.form.FailureUI(self.__tr("<b>Unable to add file '%1' to file list (file not found or insufficient permissions).</b><p>Check the file name and try again.").arg(path))
 
         self.updateFileList()
 
@@ -297,19 +304,19 @@ class ScrollPrintView(ScrollView):
         temp.reverse()
         last_item = None
         selected_item = None
-        order = len(temp)
+        index = len(temp)-1
 
         for path, mime_type, desc in temp:
-            i = FileListViewItem(self.fileListView, str(order), os.path.basename(path), desc, path)
+            log.debug("path=%s, mime_type=%s, desc=%s, index=%d" % (path, mime_type, desc, index))
+            i = FileListViewItem(self.fileListView, index, os.path.basename(path), desc, path)
             
-            if not self.prev_selected_file_path or self.prev_selected_file_path == path:
+            if self.prev_selected_file_index == index:
                 self.fileListView.setSelected(i, True)
-                self.prev_selected_file_path = path
+                self.prev_selected_file_index = index
                 selected_item = i
                 
-            order -= 1
+            index -= 1
 
-        
         last_item = self.fileListView.firstChild()
         while last_item is not None and last_item.nextSibling():
             last_item = last_item.nextSibling()
@@ -320,8 +327,6 @@ class ScrollPrintView(ScrollView):
         self.removeFilePushButton.setEnabled(file_count > 0)
         self.printButton.setEnabled(file_count > 0)
 
-
-
     def addFile_clicked(self):
         workingDirectory = user_cfg.last_used.working_dir
 
@@ -338,15 +343,13 @@ class ScrollPrintView(ScrollView):
 
         if dlg.exec_loop() == QDialog.Accepted:
                 results = dlg.selectedFile()
-                workingDirectory = str(dlg.dir().absPath())
+                workingDirectory = unicode(dlg.dir().absPath())
                 log.debug("results: %s" % results)
                 log.debug("workingDirectory: %s" % workingDirectory)
-
                 user_cfg.last_used.working_dir = workingDirectory
 
                 if results:
-                    self.addFile(str(results))
-
+                    self.addFile(unicode(results))
 
     def removeFile_clicked(self):
         try:
@@ -359,7 +362,8 @@ class ScrollPrintView(ScrollView):
             for p, t, d in temp:
                 if p == path:
                     del self.file_list[index]
-                    self.prev_selected_file_path = ''
+                    if index == self.prev_selected_file_index:
+                        self.prev_selected_file_index = 0
                     break
                 index += 1
 
@@ -482,7 +486,7 @@ class ScrollPrintView(ScrollView):
     def pageRangeEdit_lostFocus(self):
         x = []
         try:
-            x = utils.expand_range(str(self.pageRangeEdit.text()))
+            x = utils.expand_range(unicode(self.pageRangeEdit.text()))
         except ValueError:
             log.error("Invalid page range entered.")
             self.invalid_page_range = True
@@ -551,7 +555,7 @@ class ScrollPrintView(ScrollView):
             try:
                 self.cur_device.open()
             except Error:
-                self.form.FailureUI(self.__tr("<b>Cannot print: Device is busy or not available.</b><p>Please check device and try again. [1]"))
+                self.form.FailureUI(self.__tr("<b>Cannot print: Device is busy or not available.</b><p>Please check device and try again."))
                 return
 
             if 1: # Go ahead and allow - print will be queued in CUPS if not rejecting
@@ -569,7 +573,7 @@ class ScrollPrintView(ScrollView):
                 
                 copies = int(self.copiesSpinBox.value())
                 all_pages = self.pages_button_group == 0
-                page_range = str(self.pageRangeEdit.text())
+                page_range = unicode(self.pageRangeEdit.text())
                 page_set = int(self.pageSetComboBox.currentItem())
 
                 cups.resetOptions()
@@ -626,9 +630,6 @@ class ScrollPrintView(ScrollView):
                 else:
                     self.form.close()
 
-##            else:
-##                self.form.FailureUI(self.__tr("<b>Cannot print: Device is busy or not available.</b><p>Please check device and try again. [2]"))
-
         finally:
             self.cur_device.close()
 
@@ -639,4 +640,4 @@ class ScrollPrintView(ScrollView):
             self.form.close()
 
     def __tr(self,s,c = None):
-        return qApp.translate("DevMgr4",s,c)
+        return qApp.translate("ScrollPrintView",s,c)
