@@ -19,16 +19,24 @@
 # Author: Don Welch
 
 # Std Lib
-import sys, socket, re, gzip, time
+import sys
+import socket
+import re
+import gzip
+import time
 import os.path, os
+import operator
 
 # Local
 from base.g import *
-from base import msg, device, utils, service
+from base import device, utils
 from prnt import cups
+from installer import core_install
+from ui_utils import load_pixmap
 
 try:
     from fax import fax
+    #from fax import fax
     fax_import_ok = True
 except ImportError:
     # This can fail on Python < 2.3 due to the datetime module
@@ -70,16 +78,18 @@ class PrinterNameValidator(QValidator):
         QValidator.__init__(self, parent, name)
 
     def validate(self, input, pos):
-        input = str(input)
+        input = unicode(input)
         
         if not input:
             return QValidator.Acceptable, pos
 
-        elif input[pos-1] in ' #/%':
+        elif input[pos-1] in u"""~`!@#$%^&*()-=+[]{}()\\/,.<>?'\";:| """:
             return QValidator.Invalid, pos
 
-        elif input != utils.printable(input):
-            return QValidator.Invalid, pos
+        # TODO: How to determine if unicode char is "printable" and acceptable
+        # to CUPS?
+        #elif input != utils.printable(input):
+        #    return QValidator.Invalid, pos
 
         else:
             return QValidator.Acceptable, pos
@@ -91,12 +101,12 @@ class PhoneNumValidator(QValidator):
         QValidator.__init__(self, parent, name)
 
     def validate(self, input, pos):
-        input = str(input)
+        input = unicode(input)
         
         if not input:
             return QValidator.Acceptable, pos
         
-        elif input[pos-1] not in '0123456789-(+) ':
+        elif input[pos-1] not in u'0123456789-(+) ':
             return QValidator.Invalid, pos
         
         else:
@@ -115,32 +125,34 @@ class SetupForm(SetupForm_base):
         if bus is None:
             self.bus = 'usb'
         else:
-            self.bus = bus
+            self.bus = bus[0]
+            self.start_page = self.ProbedDevicesPage
+            
+        if not prop.par_build:
+            self.parRadioButton.setEnabled(False)
+            
+        if not prop.net_build:
+            self.netRadioButton.setEnabled(False)
+            
+        if not prop.par_build and not prop.net_build:
+            self.bus = 'usb'
             self.start_page = self.ProbedDevicesPage
 
         self.param = param
         self.jd_port = jd_port
 
         if self.param:
-            self.start_page = self.PPDPage
+            # Validate param...
+            device_uri, sane_uri, fax_uri = device.makeURI(self.param, self.jd_port)
 
-        icon = QPixmap(os.path.join(prop.image_dir, 'HPmenu.png'))
-        self.setIcon(icon)
+            if device_uri:
+                self.device_uri = device_uri            
+                self.start_page = self.PPDPage
+            
+            else:
+                self.FailureUI(self.__tr("<b>Device not found.</b> <p>Please make sure your printer is properly connected and powered-on."))
 
-        self.hpiod_sock = None
-        try:
-            self.hpiod_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            self.hpiod_sock.connect((prop.hpiod_host, prop.hpiod_port))
-        except socket.error:
-            log.error("Unable to connect to hpiod.")
-            raise Error(ERROR_UNABLE_TO_CONTACT_SERVICE)
-
-        self.hpssd_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            self.hpssd_sock.connect((prop.hpssd_host, prop.hpssd_port))
-        except socket.error:
-            print "Unable to connect to HPLIP I/O (hpssd)."
-            raise Error(ERROR_UNABLE_TO_CONTACT_SERVICE)
+        self.setIcon(load_pixmap('prog', '48x48'))
 
         self.connectionTypeButtonGroup.setButton(0)
         self.device_uri = ''
@@ -151,13 +163,15 @@ class SetupForm(SetupForm_base):
         self.ppd_list = []
         self.location = ''
         self.desc = ''
-        self.filter = 'none'
+        self.filter = []
         self.search = ''
         self.ttl = 4
         self.timeout = 5
-        self.printer_name_ok = False
-        self.fax_name_ok = False
+        self.printer_name_ok = True
+        self.fax_name_ok = True
         self.fax_number = ''
+        self.fax_name = ''
+        self.printer_fax_names_same = False
         self.fax_name_company = ''
         self.fax_location = ''
         self.fax_desc = ''
@@ -178,7 +192,7 @@ class SetupForm(SetupForm_base):
         self.printerNameLineEdit.setMaxLength(50)
 
         QToolTip.add(self.searchFiltersPushButton2,
-            self.__tr('Current: Filter: "%2"  Search: "%3"  TTL: %4  Timeout: %5s').arg(self.filter).arg(self.search or '').arg(self.ttl).arg(self.timeout))
+            self.__tr('Current: Filter: [%2]  Search: "%3"  TTL: %4  Timeout: %5s').arg(','.join(self.filter)).arg(self.search or '').arg(self.ttl).arg(self.timeout))
 
 
     def showPage(self, page):
@@ -198,25 +212,49 @@ class SetupForm(SetupForm_base):
         if page is self.ConnectionPage: # start --> ConnectionPage
             pass
 
-        elif page is self.ProbedDevicesPage: # ConnectionPage --> ProbedDevicesPage/EnterIPPage/DeviceNotFoundPage
+        elif page is self.ProbedDevicesPage: 
+            # ConnectionPage --> ProbedDevicesPage/EnterIPPage/DeviceNotFoundPage
             devices_found = self.updateProbedDevicesPage()
+            
 
         elif page is self.PPDPage: # ProbedDevicesPage --> PPDPage
             if self.param:
-                device_uri, sane_uri, fax_uri = device.makeURI(self.hpiod_sock, self.param, self.jd_port)
+                device_uri, sane_uri, fax_uri = device.makeURI(self.param, self.jd_port)
 
                 if device_uri:
                     self.device_uri = device_uri
-                    back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(self.device_uri)
-                    self.bus = bus
-                    self.mq = device.queryModelByURI(self.device_uri)
+            
+            back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(self.device_uri)
+            self.bus = bus
+            self.mq = device.queryModelByURI(self.device_uri)
+                    
+            norm_model = device.normalizeModelName(model).lower()
+                    
+            plugin = self.mq.get('plugin', PLUGIN_NONE)
+            if plugin > PLUGIN_NONE:
+                
+                plugin_lib = self.mq.get("plugin-library")
+                fw_download = self.mq.get("fw-download")
+                
+                self.core = core_install.CoreInstall()
+                if not self.core.check_for_plugin(norm_model):
+                    
+                    from pluginform import PluginForm
+                    plugin_form = PluginForm(self.core, norm_model, plugin, self.device_uri, plugin_lib, fw_download)
+                    ok = plugin_form.exec_loop()
+                    if not ok:
+                        self.reject()
+                        return
+                    
+                else:
+                    log.debug("Plugin support already installed")
 
             self.updatePPDPage()
 
         elif page is self.PrinterNamePage:
             self.setDefaultPrinterName()
 
-            if fax_import_ok and self.mq.get('fax-type', FAX_TYPE_NONE) != FAX_TYPE_NONE:
+            if fax_import_ok and prop.fax_build and self.mq.get('fax-type', FAX_TYPE_NONE) != FAX_TYPE_NONE:
                 self.faxCheckBox.setEnabled(True)
                 self.faxCheckBox.setEnabled(True)
                 self.faxNameLineEdit.setEnabled(True)
@@ -311,12 +349,13 @@ class SetupForm(SetupForm_base):
     def settingsDlg(self):
         dlg = SetupSettings(self.bus, self.filter, self.search, self.ttl, self.timeout, self)
         if dlg.exec_loop() == QDialog.Accepted:
+            #self.filter = [x.lower().strip() for x in dlg.filter.split(',')]
             self.filter = dlg.filter
             self.search = dlg.search
             self.ttl = dlg.ttl
             self.timeout = dlg.timeout
 
-            t = self.__tr('Current Settings: Filter: "%2"  Search: "%3"  TTL: %4  Timeout: %5s').arg(self.filter).arg(self.search or '').arg(self.ttl).arg(self.timeout)
+            t = self.__tr('Current Settings: Filter: [%2]  Search: "%3"  TTL: %4  Timeout: %5s').arg(','.join(self.filter)).arg(self.search or '').arg(self.ttl).arg(self.timeout)
 
             QToolTip.add(self.searchFiltersPushButton2, t)
             QToolTip.add(self.searchFiltersPushButton, t)
@@ -340,7 +379,7 @@ class SetupForm(SetupForm_base):
         elif self.bus == 'par':
             io_str = self.__tr("parallel port")
 
-        QToolTip.add(self.searchFiltersPushButton, self.__tr('Current Settings: Filter: "%2"  Search: "%3"  TTL: %4  Timeout: %5s').arg(self.filter).arg(self.search or '').arg(self.ttl).arg(self.timeout))
+        QToolTip.add(self.searchFiltersPushButton, self.__tr('Current Settings: Filter: [%2]  Search: "%3"  TTL: %4  Timeout: %5s').arg(','.join(self.filter)).arg(self.search or '').arg(self.ttl).arg(self.timeout))
 
         log.debug("Updating probed devices list...")
         log.debug(self.bus)
@@ -365,7 +404,23 @@ class SetupForm(SetupForm_base):
         self.probedDevicesListView.addColumn(self.__tr("Device URI"))
 
         if devices is None:
-            devices = device.probeDevices(self.hpiod_sock, self.bus, self.timeout, self.ttl, self.filter, self.search) # net_search='slp'
+            FILTER_MAP = {'print' : None,
+                          'none' : None, 
+                          'scan': 'scan-type', 
+                          'copy': 'copy-type', 
+                          'pcard': 'pcard-type',
+                          'fax': 'fax-type',
+                          }
+            
+            filter_dict = {}
+            for f in self.filter:
+                if f in FILTER_MAP:
+                    filter_dict[FILTER_MAP[f]] = (operator.gt, 0)
+                else:
+                    filter_dict[f] = (operator.gt, 0)
+                    
+            devices = device.probeDevices([self.bus], self.timeout, self.ttl, filter_dict, self.search)
+            
             self.probeHeadingTextLabel.setText(self.__tr("%1 device(s) found on the %1:").arg(len(devices)).arg(io_str))
 
         else:
@@ -442,7 +497,7 @@ class SetupForm(SetupForm_base):
         if dlg.exec_loop() == QDialog.Accepted:
             QApplication.setOverrideCursor(QApplication.waitCursor)
 
-            cups_uri, sane_uri, fax_uri = device.makeURI(self.hpiod_sock, dlg.param)
+            cups_uri, sane_uri, fax_uri = device.makeURI(dlg.param)
 
             if cups_uri:
                 back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(cups_uri)
@@ -474,9 +529,11 @@ class SetupForm(SetupForm_base):
 
         if ppds is None or not ppds:
             ppds = cups.getSystemPPDs()
+            
+        #print ppds
 
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
-        stripped_model = default_model.lower().replace('hp-', '').replace('hp_', '')
+        stripped_model = cups.stripModel(default_model)
 
         self.ppd_dict = cups.getPPDFile(stripped_model, ppds)
         log.debug(self.ppd_dict)
@@ -520,7 +577,10 @@ class SetupForm(SetupForm_base):
 
     def setDefaultPrinterName(self):
         self.installed_print_devices = device.getSupportedCUPSDevices(['hp'])
+        #self.installed_print_devices = device.getSupportedCUPSDevices('*')
         log.debug(self.installed_print_devices)
+        
+        self.installed_queues = [p.name for p in cups.getPrinters()]
 
         back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(self.device_uri)
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
@@ -545,23 +605,9 @@ class SetupForm(SetupForm_base):
         self.defaultPrinterNamePushButton.setEnabled(False)
         self.printer_name = printer_name
 
-    def printerNameLineEdit_textChanged(self,a0):
-        self.printer_name = str(a0)
-        self.defaultPrinterNamePushButton.setEnabled(True)
-
-        if not self.printer_name or (self.device_uri in self.installed_print_devices and \
-            self.printer_name in self.installed_print_devices[self.device_uri]):
-                self.setNextEnabled(self.PrinterNamePage, False)
-                self.printer_name_ok = False
-
-                if not self.printer_name:
-                    QToolTip.add(self.printerNameLineEdit, self.__tr('You must enter a name for the printer.'))
-                else:
-                    QToolTip.add(self.printerNameLineEdit, self.__tr('A printer already exists with this name. Please choose a different name.'))
-
-                self.printerNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
-
-        else:
+        
+    def setEditErrors(self):
+        if self.printer_name_ok:
             self.printerNameLineEdit.setPaletteBackgroundColor(self.bg)
             self.printer_name_ok = True
 
@@ -569,22 +615,70 @@ class SetupForm(SetupForm_base):
                 self.setNextEnabled(self.PrinterNamePage, True)
 
             QToolTip.remove(self.printerNameLineEdit)
-
-        if not self.printer_name:
-            self.printer_name_ok = False
+        
+        else:
+            self.printerNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
             self.setNextEnabled(self.PrinterNamePage, False)
+            
+        if self.fax_name_ok:
+            self.fax_name_ok = True
+            self.faxNameLineEdit.setPaletteBackgroundColor(self.bg)
+
+            if self.printer_name_ok:
+                self.setNextEnabled(self.PrinterNamePage, True)
+
+            QToolTip.remove(self.faxNameLineEdit)
+        
+        else:
+            self.faxNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
+            self.setNextEnabled(self.PrinterNamePage, False)
+    
+    
+    def printerNameLineEdit_textChanged(self,a0):
+        self.printer_name = unicode(a0)
+        self.defaultPrinterNamePushButton.setEnabled(True)
+        
+        self.printer_name_ok = True
+        
+        if not self.printer_name:
+            QToolTip.add(self.printerNameLineEdit, self.__tr('You must enter a name for the printer.'))
+            self.printer_name_ok = False
+        
+        elif self.fax_name == self.printer_name:
+            s = self.__tr('The printer name and fax name must be different. Please choose different names.')
+            QToolTip.add(self.faxNameLineEdit, s)
+            QToolTip.add(self.printerNameLineEdit, s)
+            self.fax_name_ok = False
+            self.printer_name_ok = False
+            self.printer_fax_names_same = True
+        
+        elif self.printer_name in self.installed_queues:
+            QToolTip.add(self.printerNameLineEdit, 
+                self.__tr('A printer already exists with this name. Please choose a different name.'))
+            self.printer_name_ok = False
+            
+        elif self.printer_fax_names_same:
+            if self.fax_name != self.printer_name:
+                self.printer_fax_names_same = False
+                self.printer_name_ok = True
+                
+                self.faxNameLineEdit.emit(SIGNAL("textChanged(const QString&)"), 
+                            (self.faxNameLineEdit.text(),))
+        
+        self.setEditErrors()
+
 
     def printerLocationLineEdit_textChanged(self, a0):
-        self.location = str(a0)
+        self.location = unicode(a0)
 
     def printerDescriptionLineEdit_textChanged(self,a0):
-        self.desc = str(a0)
+        self.desc = unicode(a0)
 
     def faxLocationLineEdit_textChanged(self,a0):
-        self.fax_location = str(a0)
+        self.fax_location = unicode(a0)
 
     def faxDescriptionLineEdit_textChanged(self,a0):
-        self.fax_desc = str(a0)
+        self.fax_desc = unicode(a0)
 
     def defaultPrinterNamePushButton_clicked(self):
         self.setDefaultPrinterName()
@@ -604,6 +698,7 @@ class SetupForm(SetupForm_base):
         # Check for duplicate names
         if self.fax_uri in self.installed_fax_devices and \
             fax_name in self.installed_fax_devices[self.fax_uri]:
+        #if fax_name in self.installed_queues or fax_name == self.printer_name:
                 i = 2
                 while True:
                     t = fax_name + "_%d" % i
@@ -617,40 +712,47 @@ class SetupForm(SetupForm_base):
         self.faxNameLineEdit.setPaletteBackgroundColor(self.bg)
         self.defaultFaxNamePushButton.setEnabled(False)
         self.fax_name = fax_name
+        #self.fax_name_error = False
 
     def faxNameLineEdit_textChanged(self, a0):
-        self.fax_name = str(a0)
+        self.fax_name = unicode(a0)
         self.defaultFaxNamePushButton.setEnabled(True)
-
-        if not self.fax_name or (self.fax_uri in self.installed_fax_devices and \
-            self.fax_name in self.installed_fax_devices[self.fax_uri]):
-                self.setNextEnabled(self.PrinterNamePage, False)
-                self.fax_name_ok = False
-
-                if not self.fax_name:
-                    QToolTip.add(self.faxNameLineEdit, self.__tr('You must enter a fax name.'))
-                else:
-                    QToolTip.add(self.faxNameLineEdit, self.__tr('A fax already exists with this name. Please choose a different name.'))
-
-                self.faxNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
-        else:
-            self.fax_name_ok = True
-            self.faxNameLineEdit.setPaletteBackgroundColor(self.bg)
-
-            if self.printer_name_ok:
-                self.setNextEnabled(self.PrinterNamePage, True)
-
-            QToolTip.remove(self.faxNameLineEdit)
-
+        
+        self.fax_name_ok = True
+        
         if not self.fax_name:
+            QToolTip.add(self.faxNameLineEdit, self.__tr('You must enter a fax name.'))
             self.fax_name_ok = False
-            self.setNextEnabled(self.PrinterNamePage, False)
+        
+        elif self.fax_name == self.printer_name:
+            s = self.__tr('The printer name and fax name must be different. Please choose different names.')
+            QToolTip.add(self.faxNameLineEdit, s)
+            QToolTip.add(self.printerNameLineEdit, s)
+            self.printer_name_ok = False
+            self.fax_name_ok = False
+            self.printer_fax_names_same = True
+        
+        elif self.fax_name in self.installed_queues:
+            QToolTip.add(self.faxNameLineEdit, 
+                self.__tr('A fax already exists with this name. Please choose a different name.'))
+            self.fax_name_ok = False
+            
+        elif self.printer_fax_names_same:
+            if self.fax_name != self.printer_name:
+                self.printer_fax_names_same = False
+                self.fax_name_ok = True
+                
+                self.printerNameLineEdit.emit(SIGNAL("textChanged(const QString&)"), 
+                            (self.printerNameLineEdit.text(),))
+                
+        self.setEditErrors() 
+
 
     def faxNumberLineEdit_textChanged(self, a0):
-        self.fax_number = str(a0)
+        self.fax_number = unicode(a0)
 
     def faxNameCoLineEdit_textChanged(self, a0):
-        self.fax_name_company = str(a0)
+        self.fax_name_company = unicode(a0)
 
     def faxCheckBox_clicked(self):
         pass
@@ -671,15 +773,15 @@ class SetupForm(SetupForm_base):
     def readwriteFaxInformation(self, read=True):
         try:
             QApplication.setOverrideCursor(QApplication.waitCursor)
-
-            d = fax.FaxDevice(self.fax_uri)
+            
+            d = fax.getFaxDevice(self.fax_uri, disable_dbus=True)
 
             while True:
                 try:
                     d.open()
                 except Error:
                     error_text = self.__tr("Unable to communicate with the device. Please check the device and try again.")
-                    log.error(str(error_text))
+                    log.error(unicode(error_text))
                     if QMessageBox.critical(self,
                                            self.caption(),
                                            error_text,
@@ -698,15 +800,15 @@ class SetupForm(SetupForm_base):
 
                             try:
                                 if read:
-                                    self.fax_number = d.getPhoneNum()
-                                    self.fax_name_company = d.getStationName()
+                                    self.fax_number = unicode(d.getPhoneNum())
+                                    self.fax_name_company = unicode(d.getStationName())
                                 else:
                                     d.setStationName(self.fax_name_company)
                                     d.setPhoneNum(self.fax_number)
 
                             except Error:
                                 error_text = self.__tr("<b>Device I/O Error</b><p>Could not communicate with device. Device may be busy.")
-                                log.error(str(error_text))
+                                log.error(unicode(error_text))
 
                                 if QMessageBox.critical(self,
                                                        self.caption(),
@@ -748,10 +850,10 @@ class SetupForm(SetupForm_base):
 
         #if self.ppd_file.startswith("foomatic:"):
         if not os.path.exists(self.ppd_file): # assume foomatic: or some such
-            status, status_str = cups.addPrinter(self.printer_name, self.device_uri,
+            status, status_str = cups.addPrinter(self.printer_name.encode('utf8'), self.device_uri,
                 self.location, '', self.ppd_file, self.desc)
         else:
-            status, status_str = cups.addPrinter(self.printer_name, self.device_uri,
+            status, status_str = cups.addPrinter(self.printer_name.encode('utf8'), self.device_uri,
                 self.location, self.ppd_file, '', self.desc)
 
         log.debug("addPrinter() returned (%d, %s)" % (status, status_str))
@@ -764,29 +866,72 @@ class SetupForm(SetupForm_base):
 
             self.FailureUI(self.__tr("<b>Printer queue setup failed.</b><p>Please restart CUPS and try again."))
         else:
-            service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.device_uri)
-
+            # TODO:
+            #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.device_uri)
+            pass
+            
         QApplication.restoreOverrideCursor()
 
     def setupFax(self):
         QApplication.setOverrideCursor(QApplication.waitCursor)
-
+        
+        if self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_SOAP:
+            fax_ppd_name = "HP-Fax2-hplip" # Fixed width (2528 pixels) and 300dpi rendering
+            nick = "HP Fax 2"
+        else:
+            fax_ppd_name = "HP-Fax-hplip" # Standard
+            nick = "HP Fax"
+        
         ppds = []
 
-        for f in utils.walkFiles(sys_cfg.dirs.ppd, pattern="HP*ppd*;hp*ppd*", abs_paths=True):
+        log.debug("Searching for fax file %s..." % fax_ppd_name)
+        
+        for f in utils.walkFiles(sys_cfg.dirs.ppd, pattern="HP-Fax*.ppd*", abs_paths=True):
             ppds.append(f)
 
         for f in ppds:
-            if f.find('HP-Fax') >= 0:
+            if f.find(fax_ppd_name) >= 0:
                 fax_ppd = f
                 log.debug("Found PDD file: %s" % fax_ppd)
+                log.debug("Nickname: %s" % cups.getPPDDescription(fax_ppd))
                 break
         else:
-            self.FailureUI(self.__tr("<b>Unable to find HP fax PPD file.</b><p>Please check you HPLIP installation and try again."))
             QApplication.restoreOverrideCursor()
-            return
+            log.error("Fax PPD file not found.")
+            
+            if QMessageBox.warning(self, self.__tr("Unable to find HP fax PPD file."),
+                self.__tr("The PPD file (%1.ppd) needed to setup the fax queue was not found.").arg(fax_ppd_name),
+                self.__tr("Browse to file..."), # button 0
+                self.__tr("Quit") # button 1
+                ) == 0: # Browse
+                
+                while True:
+                    fax_ppd = unicode(QFileDialog.getOpenFileName(sys_cfg.dirs.ppd, 
+                        "HP Fax PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, 
+                        "open file dialog", "Choose the fax PPD file"))
 
-        status, status_str = cups.addPrinter(self.fax_name, self.fax_uri, self.fax_location, fax_ppd, '', self.fax_desc)
+                    if not fax_ppd: # user hit cancel
+                        return
+                    
+                    if os.path.exists(fax_ppd):
+                        n = cups.getPPDDescription(fax_ppd)
+                        if n == nick:
+                            break
+                        else:
+                            self.FailureUI(self.__tr("<b>Incorrect fax PPD file.</b><p>The fax PPD file must have a nickname of '%1', not '%1'.").arg(nick).arg(n))
+                    else:
+                        self.FailureUI(self.__tr("<b>File not found.</b><p>hp-setup cannot find the file %1").arg(fax_ppd))
+                    
+            else: # Quit
+                return
+
+        if not os.path.exists(fax_ppd):
+            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'), 
+                self.fax_uri, self.fax_location, '', fax_ppd,  self.fax_desc)
+        else:        
+            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'), 
+                self.fax_uri, self.fax_location, fax_ppd, '', self.fax_desc)
+            
         log.debug("addPrinter() returned (%d, %s)" % (status, status_str))
         self.installed_fax_devices = device.getSupportedCUPSDevices(['hpfax'])
 
@@ -797,7 +942,9 @@ class SetupForm(SetupForm_base):
 
             self.FailureUI(self.__tr("<b>Fax queue setup failed.</b><p>Please restart CUPS and try again."))
         else:
-            service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.fax_uri)
+            pass
+            # TODO:
+            #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.fax_uri)
 
         QApplication.restoreOverrideCursor()
 
@@ -828,8 +975,6 @@ class SetupForm(SetupForm_base):
                         self.FailureUI(self.__tr("<b>Printer Error.</b><p>Printer is busy, offline, or in an error state. Please check the device and try again."))
                         d.close()
 
-        self.hpiod_sock.close()
-        self.hpssd_sock.close()
 
         if self.username:
             import pwd
@@ -844,13 +989,10 @@ class SetupForm(SetupForm_base):
 
 
     def reject(self):
-        self.hpiod_sock.close()
-        self.hpssd_sock.close()
-
         QWizard.reject(self)
 
     def FailureUI(self, error_text):
-        log.error(str(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", ""))
+        log.error(unicode(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", " "))
         QMessageBox.critical(self,
                              self.caption(),
                              error_text,
