@@ -19,18 +19,23 @@
 # Author: Don Welch
 #
 
+# Std Lib
+import os, os.path
+import operator
+
+# Local
 from base.g import *
 from base import utils, device
 from prnt import cups
+from ui_utils import load_pixmap
 
-import os, os.path
-
+# Qt
 from qt import *
 from scrollunload import ScrollUnloadView
 
 
 class UnloadForm(QMainWindow):
-    def __init__(self, bus='usb,par', device_uri=None, printer_name=None,
+    def __init__(self, bus=['usb', 'par'], device_uri=None, printer_name=None,
                  parent=None, name=None, fl=0):
 
         QMainWindow.__init__(self,parent,name,fl)
@@ -40,8 +45,7 @@ class UnloadForm(QMainWindow):
         self.printer_name = printer_name
         self.init_failed = False
 
-        icon = QPixmap(os.path.join(prop.image_dir, 'HPmenu.png'))
-        self.setIcon(icon)
+        self.setIcon(load_pixmap('prog', '48x48'))
 
         self.setCentralWidget(QWidget(self,"qt_central_widget"))
         self.FormLayout = QGridLayout(self.centralWidget(),1,1,11,6,"FormLayout")
@@ -56,7 +60,7 @@ class UnloadForm(QMainWindow):
             self.device_uri, self.printer_name = None, None
 
         if not self.device_uri and not self.printer_name:
-            probed_devices = device.probeDevices(None, bus=bus, filter='pcard')
+            probed_devices = device.probeDevices(bus=bus, filter={'pcard-type': (operator.eq, 1)})
             cups_printers = cups.getPrinters()
             log.debug(probed_devices)
             log.debug(cups_printers)
@@ -74,11 +78,11 @@ class UnloadForm(QMainWindow):
 
             if x == 0:
                 from nodevicesform import NoDevicesForm
-                self.failure(self.__tr("<p><b>No devices found that support photo card access.</b><p>Please make sure your device is properly installed and try again."))
+                self.FailureUI(self.__tr("<p><b>No devices found that support photo card access.</b><p>Please make sure your device is properly installed and try again."))
                 self.init_failed = True
 
             elif x == 1:
-                log.info(utils.bold("Using device: %s" % devices[0][0]))
+                log.info(log.bold("Using device: %s" % devices[0][0]))
                 self.device_uri = devices[0][0]
 
             else:
@@ -89,26 +93,28 @@ class UnloadForm(QMainWindow):
                 else:
                     self.init_failed = True
 
-        self.UnloadView = ScrollUnloadView(False, self.centralWidget(), self, "UnloadView")
+        self.dbus_avail, self.service = device.init_dbus()
+        
+        self.UnloadView = ScrollUnloadView(self.service, False, self.centralWidget(), self, "UnloadView")
         self.FormLayout.addWidget(self.UnloadView,0,0)
 
 
-        try:
-            self.cur_device = device.Device(device_uri=self.device_uri, 
-                                             printer_name=self.printer_name, 
-                                             hpssd_sock=None)
-        except Error, e:
-            log.error("Invalid device URI or printer name.")
-            self.FailureUI("<b>Invalid device URI or printer name.</b><p>Please check the parameters to hp-print and try again.")
-            self.init_failed = True
-
-        else:
-            self.device_uri = self.cur_device.device_uri
-            user_cfg.last_used.device_uri = self.device_uri
-
-            log.debug(self.device_uri)
-
-            self.statusBar().message(self.device_uri)
+        if not self.init_failed:
+            try:
+                self.cur_device = device.Device(device_uri=self.device_uri, 
+                                                 printer_name=self.printer_name)
+            except Error, e:
+                log.error("Invalid device URI or printer name.")
+                self.FailureUI("<b>Invalid device URI or printer name.</b><p>Please check the parameters to hp-print and try again.")
+                self.init_failed = True
+    
+            else:
+                self.device_uri = self.cur_device.device_uri
+                user_cfg.last_used.device_uri = self.device_uri
+    
+                log.debug(self.device_uri)
+    
+                self.statusBar().message(self.device_uri)
 
 
         QTimer.singleShot(0, self.initialUpdate)
@@ -119,11 +125,10 @@ class UnloadForm(QMainWindow):
             self.close()
             return
 
-
         self.UnloadView.onDeviceChange(self.cur_device)
 
     def FailureUI(self, error_text):
-        log.error(str(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", ""))
+        log.error(unicode(error_text).replace("<b>", "").replace("</b>", "").replace("<p>", " "))
         QMessageBox.critical(self,
                              self.caption(),
                              error_text,
